@@ -46,7 +46,8 @@ listed in the file CREDITS.
 static char *vicedir = NULL;
 
 /* Check if correct number of arguments,Too Few=1,Too Many=2,Just Right=0 */
-int check_args_num(int argc,int n){
+int check_args_num(int argc,int n)
+{
 	if(argc<n){
 		printf("Too few arguments!\n");
 		return 1;
@@ -74,10 +75,12 @@ int32_t get_id(char *n)
 }
 
 /* LOOKUP BY ID */
-void tool_byNameOrId(int argc,char *argv[]){
+void tool_byNameOrId(int argc,char *argv[])
+{
 	PDB_profile sample;
 	PDB_HANDLE h;
 	int32_t arg1;
+
 	if(check_args_num(argc,2)){
 		printf("Usage: i idnum\nidnum\t\tnumber of user/group\n");
 		return;
@@ -95,7 +98,8 @@ void tool_byNameOrId(int argc,char *argv[]){
 }
 
 /* LIST EVERTHING */
-void tool_list(int argc,char *argv[]){
+void tool_list(int argc,char *argv[])
+{
 	int32_t id;
 	PDB_profile sample;
 	PDB_HANDLE h;
@@ -353,8 +357,11 @@ void tool_update(int argc,char *argv[]){
 		return;
 	}
 	PDB_readProfile(h, arg1, &p);
-	if(p.id != 0)
-		PDB_writeProfile(h, &p);
+	if(p.id != 0) {
+		PDB_updateCps(h, &p);
+		PDB_freeProfile(&p);
+	}
+
 	PDB_db_close(h);
 }
 
@@ -463,6 +470,8 @@ again:
 
 	PDB_readProfile(h, id, &rec);
 	{
+	    if (rec.id == 0) continue;
+
 	    if (PDB_ISUSER(rec.id)) {
 		/* skip users during the second pass */
 		if (pass == 1) continue;
@@ -482,7 +491,7 @@ again:
 		fprintf(ldiffile,
 			"dn: uid=%s, %s\nobjectClass: top\n"
                         "objectClass: account\nobjectClass: posixAccount\n"
-			"cn: %s\nuid: %s\nuidNumber: %d\ngidNumber=65535\n"
+			"cn: %s\nuid: %s\nuidNumber: %d\ngidNumber: 65535\n"
 			"homeDirectory: /coda/usr/%s\n\n",
 			rec.name, basedn, rec.name, rec.name, rec.id, rec.name);
 	    } else {
@@ -560,6 +569,8 @@ void tool_export(int argc, char *argv[])
 
 	PDB_readProfile(h, id, &rec);
 	{
+	    if (rec.id == 0) continue;
+
 	    if (PDB_ISUSER(rec.id)) {
 		/* users are dumped in an /etc/passwd like format
 		 * "<username>:x:<userid>:500::/:" */
@@ -597,21 +608,31 @@ void tool_export(int argc, char *argv[])
 void tool_import(int argc, char *argv[])
 {
     FILE *userfile, *groupfile;
-    char user[64], group[64], owner_and_members[1024];
-    char *owner, *member, *s, *strtokarg = NULL;
+    char user[64], group[64], owner[64], member[64], *s;
     int32_t user_id, group_id, owner_id, member_id, create_id;
     int rc;
 
     if (check_args_num(argc, 3)) {
-	printf("Usage: import <userfile> <groupfile>\n");
+	fprintf(stderr, "Usage: import <userfile> <groupfile>\n");
+	return;
+    }
+
+    userfile = fopen(argv[1], "r");
+    if (!userfile) {
+	fprintf(stderr, "Can't open userfile '%s'\n", argv[1]);
+	return;
+    }
+
+    groupfile = fopen(argv[2], "r");
+    if (!groupfile) {
+	fprintf(stderr, "Can't open groupfile '%s'\n", argv[2]);
 	return;
     }
 
     /* recreate all users */
-    userfile = fopen(argv[1], "r");
     while(1) {
 	rc = fscanf(userfile, "%[^:]:%*[^:]:%d:%*[^\n]\n", user, &user_id);
-	if (rc < 0) break;
+	if (rc != 2) break;
 
 	if (user_id == 0) {
 	    printf("Userid 0 must be avoided, skipping entry for %s\n", user);
@@ -638,11 +659,16 @@ void tool_import(int argc, char *argv[])
     fclose(userfile);
     
     /* recreate groups */
-    groupfile = fopen(argv[2], "r");
     while (1) {
-	rc = fscanf(groupfile, "%[^:]:%*[^:]:%d:%[^\n]\n",
-		    group, &group_id, owner_and_members);
-	if (rc < 0) break;
+	int c;
+	rc = fscanf(groupfile, "%[^:]:%*[^:]:%d:%[^,\n]",
+		    group, &group_id, owner);
+	if (rc != 3) break;
+
+	/* skip to the next newline */
+	do {
+	  c = fgetc(groupfile);
+	} while (c != EOF && c != (int)'\n');
 
 	if (group_id == 0) {
 	    printf("Groupid 0 must be avoided, skipping entry for %s\n",
@@ -653,13 +679,11 @@ void tool_import(int argc, char *argv[])
 	/* restore the :'s in the group name */
 	s = group; while ((s = strchr(s, '%')) != NULL) *s = ':';
 
-	owner = strtok(owner_and_members, ",");
-
 	/* negate positive group ids, Coda groups are negative numbers */
 	if (group_id > 0) {
 	    group_id = -group_id;
 	    /* assuming this is the /etc/group file, force owner to System */
-	    owner = "System";
+	    strcpy(owner, "System");
 	}
 
 	/* create group */
@@ -689,9 +713,8 @@ void tool_import(int argc, char *argv[])
      * why we needed to create all the groups first */
     rewind(groupfile);
     while (1) {
-	rc = fscanf(groupfile, "%[^:]:%*[^:]:%d:%[^\n]\n",
-		    group, &group_id, owner_and_members);
-	if (rc < 0) break;
+	rc = fscanf(groupfile, "%[^:]:%*[^:]:%d:", group, &group_id);
+	if (rc != 2) break;
 
 	if (group_id == 0) continue;
 
@@ -700,16 +723,14 @@ void tool_import(int argc, char *argv[])
 
 	if (group_id > 0) {
 	    group_id = -group_id;
-	    strtokarg = owner_and_members;
 	} else {
 	    /* skip the owner when the group_id is negative */
-	    (void)strtok(owner_and_members, ",");
+	    fscanf(groupfile, "%[^,\n]%*[,]", owner);
 	}
 
 	/* add group members */
 	printf("Adding members to %s\n\t", group);
-	while ((member = strtok(strtokarg, ",")) != NULL) {
-	    strtokarg = NULL;
+	while (fscanf(groupfile, "%[^,\n]%*[,]", member) == 1) {
 	    /* restore the :'s in the name */
 	    s = member; while ((s = strchr(s, '%')) != NULL) *s = ':';
 
@@ -722,6 +743,7 @@ void tool_import(int argc, char *argv[])
 	    PDB_addToGroup(member_id, group_id);
 	    printf(" %s", member);
 	}
+	fgetc(groupfile); /* eat the '\n' */
 	printf("\n");
     }   
     fclose(groupfile);
@@ -814,12 +836,11 @@ ReadConfigFile()
     /* Load server.conf configuration file to get vice dir. */
     codaconf_init("server.conf");
 
-    CONF_STR(vicedir, "vicedir", "/vice");
+    CODACONF_STR(vicedir, "vicedir", "/vice");
 
     vice_dir_init(vicedir, 0);
 }
 
-
 int main(int argc, char **argv)
 {
 	int i;
@@ -840,5 +861,6 @@ int main(int argc, char **argv)
 		}
 		execute_line(line);
 	}
+	PDB_db_release();
 	return 0;
 }
