@@ -32,7 +32,8 @@ void printAcl(struct Acl *);
 /* Assumes pathname is the path of a conflict
  * Fills in repv with repvol created in beginning repair
  * Returns 0 on success, -1 on error and fills in msg if non-NULL */
-int BeginRepair(char *pathname, struct repvol **repv, char *msg, int msgsize) {
+int BeginRepair(char *pathname, struct repvol **repv, char *msg, int msgsize)
+{
     char msgbuf[DEF_BUF], space[DEF_BUF], cmd[32];
     struct ViceIoctl vioc;
     int rc;
@@ -193,9 +194,9 @@ int CompareDirs(struct repvol *repv, char *fixfile, struct repinfo *inf, char *m
     char realm[MAXHOSTNAMELEN];
     int ret, i, nreps, nConflicts = 0, setmode = 0, setacl = 0, setowner = 0;
     unsigned long j;
-    resreplica *dirs;
+    resreplica *dirs = NULL;
     struct volrep *rwv;
-    struct  listhdr *k;
+    struct  listhdr *k = NULL;
     FILE *file;
 
     if ((repv == NULL) || (inf == NULL)) {
@@ -788,8 +789,9 @@ static void growarray(char ***arrayaddr, int *arraysize) {
 
 /*  Appends expanded entried to (ASCII) fixfile
  *  Returns 0 on success, -1 on failure */
-int glexpand(char *rodir, char *fixfile, char *msg, int msgsize) {
-    int rc, gls = 0, lineno = 0;
+int glexpand(char *rodir, char *fixfile, char *msg, int msgsize)
+{
+    int rc, lineno = 0, gls = 0;
     unsigned int j, k, cnt;
     struct in_addr *custodians;
     VolumeId *repID, sID[VSG_MEMBERS];
