@@ -191,7 +191,7 @@ int getunixdirreps (int nreplicas, char *names[], resreplica **reps)
   int i,j;
   
   /* allocate space for making replica headers */
-  *reps = dirs = (resreplica *)malloc(nreplicas * sizeof(resreplica));
+  *reps = dirs = (resreplica *)calloc(nreplicas, sizeof(resreplica));
 
   /* get approximate size of directories */
   if (stat(names[0], &buf)){
@@ -345,8 +345,8 @@ int InsertListHdr (struct repair *rep, struct listhdr **ops, int index)
     repList = (struct repair *)malloc(sizeof(struct repair) * (size + 1));
     if (repList == 0) return -1;
     if (size > 0) 
-	memmove((void *) repList, (const void *)(*ops)[index].repairList, (size * sizeof(struct repair))); 
-    memmove((void *)&(repList[size]), (const void *)rep, sizeof(struct repair)); 
+	memcpy(repList, (*ops)[index].repairList, (size * sizeof(struct repair))); 
+    memcpy(&(repList[size]), rep, sizeof(struct repair)); 
     /*    free ((*ops)[index].repairList);  */
     ((*ops)[index]).repairList = repList; 
     ((*ops)[index]).repairCount ++; 
@@ -645,7 +645,10 @@ void resClean (int nreplicas, resreplica *dirs, struct listhdr *lh)
 	free(lh);
     }
     /* free up the global array of dir entries */
-    free(direntriesarr);
+    if (direntriesarr) {
+	free(direntriesarr);
+	direntriesarr = NULL;
+    }
 }
 
 int GetParent(char *realm, ViceFid *cfid, ViceFid *dfid, char *volmtpt, char *dpath, char *childname) {
