@@ -161,20 +161,9 @@ void CommInit() {
     RPC2_Perror = 0;
 
     /* Port initialization. */
-    struct servent *s;
     RPC2_PortIdent port1;
     port1.Tag = RPC2_PORTBYINETNUMBER;
-
-    if (masquerade) {
-        port1.Value.InetPortNumber = htons(masquerade_port);
-    } else {
-        s = getservbyname("venus", "udp");
-        if (s) port1.Value.InetPortNumber = s->s_port;
-        else {
-	    eprint("getservbyname(venus,udp) failed, using 2430/udp");
-	    port1.Value.InetPortNumber = htons(2430);
-	}
-    }
+    port1.Value.InetPortNumber = htons(masquerade_port);
 
     /* SFTP initialization. */
     SFTP_Initializer sei;
@@ -184,19 +173,7 @@ void CommInit() {
     sei.AckPoint = sftp_ackpoint;
     sei.PacketSize = sftp_packetsize;
     sei.EnforceQuota = 1;
-
     sei.Port.Tag = (enum PortTag)0;
-    if (!masquerade) {
-        sei.Port.Tag = RPC2_PORTBYINETNUMBER;
-
-        s = getservbyname("venus-se", "udp");
-        if (s) sei.Port.Value.InetPortNumber = s->s_port;
-        else {
-	    eprint("getservbyname(venus-se,udp) failed, using 2431/udp");
-	    sei.Port.Value.InetPortNumber = htons(2431);
-	}
-    }
-
     SFTP_Activate(&sei);
 
     /* RPC2 initialization. */
@@ -220,31 +197,6 @@ void CommInit() {
 
 /* *****  Connection  ***** */
 
-const int MAXCONNSPERUSER = 9;		    /* Max simultaneous conns per user per server. */
-
-#define	CONNQ_LOCK()
-#define	CONNQ_UNLOCK()
-#define	CONNQ_WAIT()	    VprocWait((char *)&connent::conntab_sync)
-#define	CONNQ_SIGNAL()	    VprocSignal((char *)&connent::conntab_sync)
-
-void Conn_Wait() {
-    CONNQ_LOCK();
-    LOG(0, ("WAITING(CONNQ):\n"));
-    START_TIMING();
-    CONNQ_WAIT();
-    END_TIMING();
-    LOG(0, ("WAIT OVER, elapsed = %3.1f\n", elapsed));
-    CONNQ_UNLOCK();
-}
-
-
-void Conn_Signal() {
-    CONNQ_LOCK();
-    CONNQ_SIGNAL();
-    CONNQ_UNLOCK();
-}
-
-
 int srvent::GetConn(connent **cpp, uid_t uid, int Force)
 {
     LOG(100, ("srvent::GetConn: host = %s, uid = %d, force = %d\n",
@@ -255,8 +207,7 @@ int srvent::GetConn(connent **cpp, uid_t uid, int Force)
     connent *c = 0;
 
     /* Grab an existing connection if one is free. */
-    /* Before creating a new connection, make sure the per-user limit is not exceeded. */
-    for (;;) {
+    {
 	/* Check whether there is already a free connection. */
 	struct ConnKey Key; Key.host = host; Key.uid = uid;
 	conn_iterator next(&Key);
@@ -269,15 +220,6 @@ int srvent::GetConn(connent **cpp, uid_t uid, int Force)
                 return 0;
 	    }
 	}
-
-	if (count < MAXCONNSPERUSER) break;
-
-	/* Wait here if MAX conns are already in use. */
-	/* Synchronization needs fixed for MP! -JJK */
-	if (VprocInterrupted()) return(EINTR);
-/*	    RPCOpStats.RPCOps[opcode].busy++;*/
-	Conn_Wait();
-	if (VprocInterrupted()) return(EINTR);
     }
 
     /* Try to connect to the server on behalf of the user. */
@@ -326,8 +268,6 @@ void PutConn(connent **cpp)
     else {
 	c->inuse = 0;
     }
-
-    Conn_Signal();
 }
 
 
@@ -501,6 +441,7 @@ connent *conn_iterator::operator()() {
     olink *o;
     while ((o = olist_iterator::operator()())) {
 	connent *c = strbase(connent, o, tblhandle);
+	if (c->dying) continue;
 	if (key == (struct ConnKey *)0) return(c);
 	if ((key->host.s_addr == c->srv->host.s_addr ||
              key->host.s_addr == INADDR_ANY) &&
@@ -576,18 +517,22 @@ srvent *FindServerByCBCid(RPC2_Handle connid)
 }
 
 
-srvent *GetServer(struct in_addr *host, RealmId realm)
+srvent *GetServer(struct in_addr *host, RealmId realmid)
 {
     CODA_ASSERT(host && host->s_addr);
     LOG(100, ("GetServer: host = %s\n", inet_ntoa(*host)));
 
     srvent *s = FindServer(host);
     if (s) {
-        s->GetRef();
-	return s;
+	s->GetRef();
+	if (s->realmid == realmid)
+	    return s;
+
+	s->Reset();
+	PutServer(&s);
     }
 
-    s = new srvent(host, realm);
+    s = new srvent(host, realmid);
 
     srvent::srvtab->insert(&s->tblhandle);
 
@@ -990,7 +935,7 @@ srvent::srvent(struct in_addr *Host, RealmId realm)
     bwmax  = INIT_BW;
     lastobs.tv_sec = lastobs.tv_usec = 0;
     VGAPlusSHA_Supported = 0;  /* default is old-style server */
-    
+    refcount = 1;
 
 #ifdef	VENUSDEBUG
     allocs++;
@@ -1037,9 +982,8 @@ int srvent::Connect(RPC2_Handle *cidp, int *authp, uid_t uid, int Force)
     /* Get the user entry and attempt to connect to it. */
     Xbinding = 1;
     {
-	userent *u = 0;
 	Realm *realm = REALMDB->GetRealm(realmid);
-	GetUser(&u, realm, uid);
+	userent *u = realm->GetUser(uid);
 	code = u->Connect(cidp, authp, &host);
 	PutUser(&u);
 	realm->PutRef();
