@@ -33,7 +33,6 @@ listed in the file CREDITS.
 class fsdb;
 class fsobj;
 class fso_iterator;
-class fso_vol_iterator;
 
 class cmlent;			    /* we have compiler troubles if volume.h is included! */
 class lrdb;
@@ -393,7 +392,6 @@ class fsobj {
   friend int FSO_PriorityFN(bsnode *, bsnode *);
   friend class fsdb;
   friend class fso_iterator;
-  friend class fso_vol_iterator;
   friend long VENUS_CallBackFetch(RPC2_Handle, ViceFid *, SE_Descriptor *);
   friend class vproc;
   friend class namectxt;
@@ -418,7 +416,7 @@ class fsobj {
 
     /* Links for various lists. */
     rec_olink primary_handle;			/* link for {fstab, free-list} */
-    /*T*/olink vol_handle;                      /* link for volent fso_list */
+    /*T*/struct dllist_head vol_handle;         /* link for volent fso_list */
     /*T*/bsnode prio_handle;			/* link for priority queue */
     /*T*/dlink del_handle;			/* link for delete queue */
     /*T*/olink owrite_handle;			/* link for owrite queue */
@@ -504,10 +502,9 @@ class fsobj {
     void Kill(int =1);
     void GC();
     int Flush();
-    void UpdateStatusAndSHA(ViceStatus *, uid_t, RPC2_BoundedBS *);
-    void UpdateStatusAndClearSHA(ViceStatus *, ViceVersionVector *, uid_t);
+    void UpdateStatus(ViceStatus *, ViceVersionVector *, uid_t);
     int StatusEq(ViceStatus *, int);
-    void ReplaceStatusAndSHA(ViceStatus *, ViceVersionVector *, RPC2_BoundedBS *);
+    void ReplaceStatus(ViceStatus *, ViceVersionVector *);
     int CheckRcRights(int);
     void SetRcRights(int);
     void ClearRcRights();
@@ -642,6 +639,8 @@ class fsobj {
 			  char *, unsigned short, int);
     int DisconnectedSymlink(Date_t, uid_t, fsobj **, char *,
 			     char *, unsigned short, int, int);
+    int GetContainerFD(void);
+    int LookAside(void);
 
   public:
     /* The public CFS interface (Vice portion). */
@@ -690,7 +689,7 @@ class fsobj {
     int IsRoot() { return(mvstat == ROOT); }
     int IsVenusRoot() { return(FID_EQ(&fid, &rootfid)); }
     int	IsMtPt() { return(mvstat == MOUNTPOINT); }      /* covered mount point */
-    int	IsMTLink() { return(stat.VnodeType == (int)SymbolicLink && stat.Mode == 0644); }
+    int	IsMTLink() { return(IsSymLink() && stat.Mode == 0644 && IsNormal()); }
                                                         /* uncovered mount point */
     int IsBackFetching();			/* fso involved in an ongoing reintegration */
     int SetLastResolved(long t) { lastresolved = t; return(0); }
@@ -750,14 +749,6 @@ class fso_iterator : public rec_ohashtab_iterator {
     fsobj *operator()();
 };
 
-class fso_vol_iterator : public olist_iterator {
-    LockLevel clevel;
-
- public:
-    fso_vol_iterator(LockLevel, volent *);
-    fsobj *operator()();
-};
-
 /*  *****  Variables  ***** */
 
 extern int CacheFiles;
