@@ -667,15 +667,11 @@ RestartFind:
 	f->Lock(RD);
 
 	/* Update component. */
-	if (comp && comp[0] != '\0' && !STREQ(comp, ".") && !STREQ(comp, "..") &&
-	    !STREQ(comp, f->comp)) {
+	if (comp && comp[0] != '\0' && !STREQ(comp, ".") && !STREQ(comp, "..") && !STREQ(comp, f->comp)) {
 		Recov_BeginTrans();
 		RVMLIB_REC_OBJECT(f->comp);
 		rvmlib_rec_free(f->comp);
-		int len = (int) strlen(comp) + 1;
-		f->comp = (char *)rvmlib_rec_malloc(len);
-		rvmlib_set_range(f->comp, len);
-		strcpy(f->comp, comp);
+		f->comp = rvmlib_strdup(comp);
 		Recov_EndTrans(MAXFP);
 	}
     }
@@ -854,8 +850,12 @@ RestartFind:
                  * the fetch will modify f->cf.ValidData) */
                 nblocks -= NBLOCKS(f->cf.ValidData());
 
-                /* Let fsobj::Fetch go ahead and fetch the object */
-                code = f->Fetch(uid);
+		code = 0;
+		/* first try the LookAside cache */
+		if (!f->LookAside()) {
+		    /* Let fsobj::Fetch go ahead and fetch the object */
+		    code = f->Fetch(uid);
+		}
 
                 /* Restart operation in case of inconsistency. */
                 if (code == EINCONS)
@@ -914,9 +914,19 @@ RestartFind:
                     Put(&f);
                     return(ETOOMANYREFS);
                 } 
+
                 if (!HAVEALLDATA(f)) {
-                    Put(&f);
-                    return(ETIMEDOUT);
+		    int found;
+
+		    /* try the lookaside cache */
+		    f->PromoteLock();
+		    found = f->LookAside();
+		    f->DemoteLock();
+
+		    if (!found) {
+			Put(&f);
+			return(ETIMEDOUT);
+		    }
                 }
 
                 if (!DATAVALID(f))
@@ -1035,15 +1045,23 @@ void fsdb::Flush(Volid *vid)
     /* comment in fsdb::Flush applies here */
     int restart = 1;
     while (restart) {
-	fsobj *f;
-	fso_vol_iterator next(NL, v);
-	
+	struct dllist_head *p, *next;
 	restart = 0;
-	while ((f = next())) 
-	    if (f->Flush() == 0) {
-		restart = 1;
-		break;
+
+	for(p = v->fso_list.next; p != &v->fso_list; p = next) {
+	    fsobj *n = NULL, *f = list_entry_plusplus(p, fsobj, vol_handle);
+	    next = p->next;
+
+	    if (next != &v->fso_list) {
+		n = list_entry_plusplus(next, fsobj, vol_handle);
+		FSO_HOLD(n);
 	    }
+	
+	    if (f->Flush() == 0)
+		restart = 1;
+
+	    if (n) FSO_RELE(n);
+	}
     }
     v->release();
 }
@@ -1335,8 +1353,8 @@ int fsdb::DirtyBlockCount() {
     fso_iterator next(NL);
     fsobj *f;
     while ((f = next())) {
-        if (!REPLACEABLE(f)) {
-	    count += BLOCKS(f);
+        if (!REPLACEABLE(f) && !f->IsSymLink()) {
+	    count += NBLOCKS(f->cf.Length());
 	}
     }
 
