@@ -178,6 +178,7 @@ void fsobj::ResetPersistent() {
     CleanStat.Length = (unsigned long)-1;
     CleanStat.Date = (Date_t)-1;
     data.havedata = 0;
+    memset(VenusSHA, 0, SHA_DIGEST_LENGTH);
 }
 
 /* local-repair modification */
@@ -189,12 +190,12 @@ void fsobj::ResetTransient()
 	{ print(logFile); CHOKE("fsobj::ResetTransient: bogus MagicNumber"); }
 
     /* This is a horrible way of resetting handles! */
-    memset((void *)&vol_handle, 0, (int)sizeof(vol_handle));
+    list_head_init(&vol_handle);
     memset((void *)&prio_handle, 0, (int)sizeof(prio_handle));
     memset((void *)&del_handle, 0, (int)sizeof(del_handle));
     memset((void *)&owrite_handle, 0, (int)sizeof(owrite_handle));
 
-    if (HAVEDATA(this) && stat.VnodeType == Directory && mvstat != MOUNTPOINT) {
+    if (HAVEDATA(this) && IsDir()) {
 	data.dir->udcfvalid = 0;
 	data.dir->udcf = 0;
     }
@@ -240,7 +241,7 @@ void fsobj::ResetTransient()
     }
 
     /* Add to volume list */
-    vol->fso_list->append(&vol_handle);
+    list_add(&vol_handle, &vol->fso_list);
 
     if (IsLocalObj()) {
 	/* set valid RC status for local object */
@@ -345,8 +346,7 @@ fsobj::~fsobj() {
     }
 
     /* Remove from volume's fso list */
-    if (vol->fso_list->remove(&vol_handle) != &vol_handle)
-	{ print(logFile); CHOKE("fsobj::~fsobj: fso_list remove"); }
+    list_del(&vol_handle);
 
     /* Unlink from volume. */
     VDB->Put(&vol);
@@ -410,10 +410,14 @@ void fsobj::Recover()
     }
 
     /* Uncover mount points. */
-    if (mvstat == MOUNTPOINT) {
+    if (IsMtPt()) {
 	Recov_BeginTrans();
+
+	/* XXX this can probably be removed */
 	RVMLIB_REC_OBJECT(stat.VnodeType);
 	stat.VnodeType = SymbolicLink;
+	/* XXX */
+
 	RVMLIB_REC_OBJECT(mvstat);
 	mvstat = NORMAL;
 	Recov_EndTrans(MAXFP);
@@ -477,7 +481,7 @@ void fsobj::Recover()
 
     /* Get rid of a former mount-root whose fid is not a volume root and whose
      * pfid is NullFid */
-    if ((mvstat == NORMAL) && !FID_IsVolRoot(&fid) && 
+    if (IsNormal() && !FID_IsVolRoot(&fid) && 
 	FID_EQ(&pfid, &NullFid) && !IsLocalObj()) {
 	LOG(0, ("fsobj::Recover: (%s) is a non-volume root whose pfid is NullFid\n",
 		FID_(&fid)));
@@ -579,6 +583,7 @@ void fsobj::Matriculate()
 /* Need not be called from within transaction. */
 /* Call with object write-locked. */
 /* CallBack handler calls this with NoLock (to avoid deadlock)! -JJK */
+/* >> Demote should not yield or destroy the object << */
 void fsobj::Demote(void)
 {
     if (!HAVESTATUS(this) || DYING(this)) return;
@@ -663,27 +668,24 @@ int fsobj::Flush() {
 /* MUST be called from within transaction! */
 /* Call with object write-locked. */
 /* Called as result of {GetAttr, ValidateAttr, GetData, ValidateData}. */
-
-void fsobj::UpdateStatusAndSHA(ViceStatus *vstat, uid_t uid, RPC2_BoundedBS *newsha)
+void fsobj::UpdateStatus(ViceStatus *vstat, vv_t *UpdateSet, uid_t uid)
 {
     /* Mount points are never updated. */
     if (IsMtPt())
-	{ print(logFile); CHOKE("fsobj::UpdateStatusAndSHA: IsMtPt!"); }
+	{ print(logFile); CHOKE("fsobj::UpdateStatus: IsMtPt!"); }
     /* Fake objects are never updated. */
     if (IsFake())
-	{ print(logFile); CHOKE("fsobj::UpdateStatusAndSHA: IsFake!"); }
+	{ print(logFile); CHOKE("fsobj::UpdateStatus: IsFake!"); }
 
-    LOG(100, ("fsobj::UpdateStatusAndSHA: (%s), uid = %d\n", FID_(&fid), uid));
+    LOG(100, ("fsobj::UpdateStatus: (%s), uid = %d\n", FID_(&fid), uid));
 
     if (HAVESTATUS(this)) {		/* {ValidateAttr} */
       if (!StatusEq(vstat, 0))
-	    ReplaceStatusAndSHA(vstat, 0, newsha);
-      /* else ValidateAttr was successful for this object, so
-	 leave SHA alone */
+	    ReplaceStatus(vstat, UpdateSet);
     }
     else {				/* {GetAttr} */
+	ReplaceStatus(vstat, UpdateSet);
 	Matriculate();
-	ReplaceStatusAndSHA(vstat, 0, newsha);
     }
 
     /* Set access rights and parent (if they differ). */
@@ -694,36 +696,6 @@ void fsobj::UpdateStatusAndSHA(ViceStatus *vstat, uid_t uid, RPC2_BoundedBS *new
 }
 
 
-/* MUST be called from within transaction! */
-/* Call with object write-locked. */
-/* Called for mutating operations. */
-/* SHA is always cleared by this call */
-void fsobj::UpdateStatusAndClearSHA(ViceStatus *vstat, vv_t *UpdateSet, uid_t uid)
-{
-    /* Mount points are never updated. */
-    if (IsMtPt())
-	{ print(logFile); CHOKE("fsobj::UpdateStatusAndClearSHA: IsMtPt!"); }
-    /* Fake objects are never updated. */
-    if (IsFake())
-	{ print(logFile); CHOKE("fsobj::UpdateStatusAndClearSHA: IsFake!"); }
-
-    LOG(100, ("fsobj::UpdateStatusAndClearSHA: (%s), uid = %d\n", FID_(&fid), uid));
-
-    /* Install the new status block. */
-    if (!StatusEq(vstat, 1))
-	/* Ought to Die in this event! */;
-
-    ReplaceStatusAndSHA(vstat, UpdateSet, NULL);
-
-    /* Set access rights and parent (if they differ). */
-    /* N.B.  It should be a fatal error if they differ! */
-    if (IsDir())
-	SetAcRights(uid, vstat->MyAccess, vstat->AnyAccess);
-
-    SetParent(vstat->vparent, vstat->uparent);
-}
-
-
 /* Need not be called from within transaction. */
 int fsobj::StatusEq(ViceStatus *vstat, int Mutating)
 {
@@ -791,20 +763,15 @@ int fsobj::StatusEq(ViceStatus *vstat, int Mutating)
 
 /* MUST be called from within transaction! */
 /* Call with object write-locked. */
-void fsobj::ReplaceStatusAndSHA(ViceStatus *vstat, vv_t *UpdateSet, RPC2_BoundedBS *vsha)
+void fsobj::ReplaceStatus(ViceStatus *vstat, vv_t *UpdateSet)
 {
     RVMLIB_REC_OBJECT(stat);
-    RVMLIB_REC_OBJECT(VenusSHA);
-
-    if (vsha && vsha->SeqLen == SHA_DIGEST_LENGTH)
-	 memcpy(VenusSHA, vsha->SeqBody, SHA_DIGEST_LENGTH);
-    else memset(VenusSHA, 0, SHA_DIGEST_LENGTH);
 
     /* We're changing the length? 
      * Then the cached data is probably no longer useable! But try to fix up
      * the cachefile so that we can at least give a stale copy. */
     if (HAVEDATA(this) && stat.Length != vstat->Length) {
-	LOG(0, ("fsobj::ReplaceStatusAndSHA: (%s), changed stat.length %d->%d\n",
+	LOG(0, ("fsobj::ReplaceStatus: (%s), changed stat.length %d->%d\n",
 		FID_(&fid), stat.Length, vstat->Length));
 	if (IsFile())
 	    LocalSetAttr((unsigned long)-1, vstat->Length, (unsigned long)-1,
@@ -829,7 +796,8 @@ void fsobj::ReplaceStatusAndSHA(ViceStatus *vstat, vv_t *UpdateSet, RPC2_Bounded
     stat.Owner = (uid_t) vstat->Owner;
     stat.Mode = (short) vstat->Mode;
     stat.LinkCount = (unsigned char) vstat->LinkCount;
-    stat.VnodeType = vstat->VnodeType;
+    if (vstat->VnodeType)
+	stat.VnodeType = vstat->VnodeType;
 }
 
 
@@ -936,8 +904,7 @@ void fsobj::SetAcRights(uid_t uid, long my_rights, long any_rights)
     AnyUser.valid = 1;
 
     /* Don't record my_rights if we're not really authenticated! */
-    userent *ue;
-    GetUser(&ue, vol->realm, uid);
+    userent *ue = vol->realm->GetUser(uid);
     int tokensvalid = ue->TokensValid();
     PutUser(&ue);
     if (!tokensvalid) return;
@@ -994,8 +961,7 @@ void fsobj::PromoteAcRights(uid_t uid)
 	 */
 	for (int i = 0; i < CPSIZE; i++)
 	    if (SpecificUser[i].inuse && !SpecificUser[i].valid) {
-		userent *ue;
-		GetUser(&ue, vol->realm, SpecificUser[i].uid);
+		userent *ue = vol->realm->GetUser(SpecificUser[i].uid);
 		int tokensvalid = ue->TokensValid();
 		PutUser(&ue);
 		if (tokensvalid) SpecificUser[i].valid = 1;
@@ -1007,8 +973,7 @@ void fsobj::PromoteAcRights(uid_t uid)
 	 * he goes disconnected, he may have access to files he 
 	 * otherwise wouldn't have because he lost tokens.
 	 */
-	userent *ue;
-	GetUser(&ue, vol->realm, uid);
+	userent *ue = vol->realm->GetUser(uid);
 	int tokensvalid = ue->TokensValid();
 	PutUser(&ue);
 	if (!tokensvalid) return;
@@ -1174,9 +1139,6 @@ int fsobj::TryToCover(VenusFid *inc_fid, uid_t uid)
 	code = VDB->Get(&tvol, vol->realm, &data.symlink[1], this);
 
     if (code != 0) {
-/*
-	 eprint("TryToCover(%s) failed (%d)", data.symlink, code);
-*/
 	LOG(100, ("fsobj::TryToCover: vdb::Get(%s) failed (%d)\n", data.symlink, code));
 	return(code);
     }
@@ -1240,7 +1202,7 @@ int fsobj::TryToCover(VenusFid *inc_fid, uid_t uid)
 
     /* Do the mount magic. */
     Recov_BeginTrans();
-    if (IsFake() && rf->mvstat != ROOT) {
+    if (IsFake() && !rf->IsRoot()) {
 	    RVMLIB_REC_OBJECT(rf->mvstat);
 	    RVMLIB_REC_OBJECT(rf->u.mtpoint);
 	    rf->mvstat = ROOT;
@@ -1259,7 +1221,7 @@ int fsobj::TryToCover(VenusFid *inc_fid, uid_t uid)
 /* MUST be called from within transaction! */
 /* Call with object write-locked. */
 void fsobj::CoverMtPt(fsobj *root_fso) {
-    if (mvstat != NORMAL)
+    if (!IsNormal())
 	{ print(logFile); CHOKE("fsobj::CoverMtPt: mvstat != NORMAL"); }
     if (!data.symlink)
 	{ print(logFile); CHOKE("fsobj::CoverMtPt: no data.symlink!"); }
@@ -1273,7 +1235,6 @@ void fsobj::CoverMtPt(fsobj *root_fso) {
     k_Purge(&fid, 1);
 
     /* Enter new state (MOUNTPOINT). */
-    stat.VnodeType = Directory;
     mvstat = MOUNTPOINT;
     u.root = root_fso;
     DisableReplacement();
@@ -1283,7 +1244,7 @@ void fsobj::CoverMtPt(fsobj *root_fso) {
 /* MUST be called from within transaction! */
 /* Call with object write-locked. */
 void fsobj::UncoverMtPt() {
-    if (mvstat != MOUNTPOINT) 
+    if (!IsMtPt()) 
 	{ print(logFile); CHOKE("fsobj::UncoverMtPt: mvstat != MOUNTPOINT"); }
     if (!u.root)
 	{ print(logFile); CHOKE("fsobj::UncoverMtPt: no u.root!"); }
@@ -1299,7 +1260,6 @@ void fsobj::UncoverMtPt() {
     k_Purge(&pfid, 1);			/* This IS necessary. */
 
     /* Enter new state (NORMAL). */
-    stat.VnodeType = SymbolicLink;
     mvstat = NORMAL;
     EnableReplacement();
 }
@@ -1308,7 +1268,7 @@ void fsobj::UncoverMtPt() {
 /* MUST be called from within transaction! */
 /* Call with object write-locked. */
 void fsobj::MountRoot(fsobj *mtpt_fso) {
-    if (mvstat != ROOT)
+    if (!IsRoot())
 	{ print(logFile); CHOKE("fsobj::MountRoot: mvstat != ROOT"); }
     if (u.mtpoint)
 	{ print(logFile); CHOKE("fsobj::MountRoot: u.mtpoint exists!"); }
@@ -1329,7 +1289,7 @@ void fsobj::MountRoot(fsobj *mtpt_fso) {
 /* MUST be called from within transaction! */
 /* Call with object write-locked. */
 void fsobj::UnmountRoot() {
-    if (mvstat != ROOT) 
+    if (!IsRoot()) 
 	{ print(logFile); CHOKE("fsobj::UnmountRoot: mvstat != ROOT"); }
     if (!u.mtpoint)
 	{ print(logFile); CHOKE("fsobj::UnmountRoot: no u.mtpoint!"); }
@@ -2007,8 +1967,10 @@ int fsobj::Fakeify()
 
 	else {
 	    /* Laboriously scan database to find our parent! */
-	    fso_vol_iterator next(NL, vol);
-	    while ((pf = next())) {
+	    struct dllist_head *p;
+	    list_for_each(p, vol->fso_list) {
+		pf = list_entry_plusplus(p, fsobj, vol_handle);
+
 		if (!pf->IsDir() || pf->IsMtPt()) continue;
 		if (!HAVEALLDATA(pf)) continue;
 		if (!pf->dir_IsParent(&fid)) continue;
@@ -2544,7 +2506,7 @@ void fsobj::CacheReport(int fd, int level) {
 
     /* Indirect over mount points. */
     if (IsMtPt()) {
-	u.mtpoint->CacheReport(fd, level);
+	u.root->CacheReport(fd, level);
 	return;
     }
 
@@ -2794,20 +2756,6 @@ fsobj *fso_iterator::operator()() {
     }
 }
 
-/* for iteration by volume */
-fso_vol_iterator::fso_vol_iterator(LockLevel level, volent *vol) : olist_iterator(*vol->fso_list) {
-    clevel = level;
-}
-
-fsobj *fso_vol_iterator::operator()() {
-    olink *o = olist_iterator::operator()();
-    if (!o) return(0);
-
-    fsobj *f = strbase(fsobj, o, vol_handle);
-    if (clevel != NL) f->Lock(clevel);
-    return(f);
-}
-
 void fsobj::GetOperationState(int *conn, int *tid)
 {
     if (HOARDING(this)) {
@@ -2834,7 +2782,7 @@ void fsobj::GetOperationState(int *conn, int *tid)
 	    if (cfo->IsLocalObj())
 	      break;
 	    if (cfo->IsRoot())
-	      cfo = cfo->u.mtpoint;
+	      cfo = cfo->u.mtpoint->pfso;
 	    else
 	      cfo = cfo->pfso;
 	}
