@@ -66,7 +66,7 @@ extern "C" {
 /* C-stub to jump into the c++ method without compiler warnings */
 static void FetchProgressIndicator_stub(void *up, unsigned int offset)
 {
-    ((fsobj *)up)->FetchProgressIndicator((long)offset);
+    ((fsobj *)up)->FetchProgressIndicator((unsigned long)offset);
 }
 
 void fsobj::FetchProgressIndicator(unsigned long offset)
@@ -74,20 +74,110 @@ void fsobj::FetchProgressIndicator(unsigned long offset)
     unsigned long last;
     unsigned long curr;
     
-    if (stat.Length == 0) return;
-
-    if (!GotThisData) last = 0;
-    else	      last = 100 / (stat.Length / GotThisData);
-
-    if (!offset) curr = 0;
-    else	 curr = 100 / (stat.Length / offset);
+    if (stat.Length > 100000) {
+        last = GotThisData / (stat.Length / 100) ; 
+	curr = offset / (stat.Length / 100) ;
+    } else if (stat.Length > 0) {
+        last = 100 * GotThisData / stat.Length ; 
+	curr = 100 * offset / stat.Length ;
+    } else {
+	last = 0;
+	curr = 100;
+    }
 
     if (last != curr)
-	MarinerLog("progress::fetching (%s) %lux\n", comp, curr);
+	MarinerLog("progress::fetching (%s) %lu%%\n", comp, curr);
 
     GotThisData = (unsigned long)offset;
 }
 
+/* MUST be called from within a transaction */
+int fsobj::GetContainerFD(void)
+{
+    FSO_ASSERT(this, IsFile());
+
+    RVMLIB_REC_OBJECT(data.file);
+
+    /* create a sparse file of the desired size */
+    if (!HAVEDATA(this)) {
+	RVMLIB_REC_OBJECT(cf);
+	data.file = &cf;
+	data.file->Create(stat.Length);
+    }
+
+    /* and open the containerfile */
+    return data.file->Open(O_WRONLY);
+}
+
+int fsobj::LookAside(void)
+{
+    long cbtemp = cbbreaks;
+    int fd = -1, lka_successful = 0;
+    char emsg[256];
+
+    CODA_ASSERT(!IsLocalObj() && !IsFake());
+    CODA_ASSERT(HAVESTATUS(this) && !HAVEALLDATA(this));
+
+    if (!IsFile() || IsZeroSHA(VenusSHA))
+	return 0;
+
+    /* (Satya, 1/03)
+       Do lookaside to see if fetch can be short-circuited.
+       Assumptions for lookaside:
+       (a) we are operating normally (not in a repair)
+       (b) we have valid status
+       (c) file is a plain file (not sym link, directory, etc.)
+       (d) non-zero SHA
+
+       These were all verified in Sanity Checks above;
+       Check for replicated volume further ensures we never
+       do lookaside during repair
+
+       SHA value is obtained initially from fsobj and used for lookaside.
+     */
+
+    Recov_BeginTrans();
+    RVMLIB_REC_OBJECT(flags);
+    flags.fetching = 1;
+    fd = GetContainerFD();
+    Recov_EndTrans(CMFP);
+
+    if (fd != -1) {
+	memset(emsg, 0, sizeof(emsg));
+
+	lka_successful = LookAsideAndFillContainer(VenusSHA, fd,
+						   stat.Length, venusRoot,
+						   emsg, sizeof(emsg)-1);
+	data.file->Close(fd);
+
+	if (emsg[0])
+	    LOG(0, ("LookAsideAndFillContainer(%s): %s\n", cf.Name(), emsg));
+
+	if (lka_successful)
+	    LOG(0, ("Lookaside of %s succeeded!\n", cf.Name()));
+    }
+
+    /* Note that the container file now has all the data we expected */
+    Recov_BeginTrans();
+    RVMLIB_REC_OBJECT(flags);
+    flags.fetching = 0;
+
+    if (lka_successful) {
+	data.file->SetLength(stat.Length);
+	cf.SetValidData(cf.Length()); 
+    }
+    Recov_EndTrans(CMFP);
+
+    /* If we received any callbacks during the lookaside, the validity of the
+     * found data is suspect and we shouldn't set the status to valid */
+    if (lka_successful && cbtemp == cbbreaks)
+	SetRcRights(RC_DATA | RC_STATUS); /* we now have the data */
+
+    /* we're done, skip out of here */
+    return lka_successful;
+}
+
+
 int fsobj::Fetch(uid_t uid)
 {
     int fd = -1;
@@ -155,17 +245,9 @@ int fsobj::Fetch(uid_t uid)
 	    sei->ByteQuota = -1;
 	    switch(stat.VnodeType) {
 		case File:
-		    RVMLIB_REC_OBJECT(data.file);
-                    
-		    /* create a sparse file of the desired size */
-                    if (!HAVEDATA(this)) {
-                        RVMLIB_REC_OBJECT(cf);
-                        data.file = &cf;
-                        data.file->Create(stat.Length);
-                    }
-
                     /* and open the containerfile */
-		    fd = data.file->Open(O_WRONLY);
+		    fd = GetContainerFD();
+		    CODA_ASSERT(fd != -1);
 
 		    sei->Tag = FILEBYFD;
 		    sei->FileInfo.ByFD.fd = fd;
@@ -224,51 +306,6 @@ int fsobj::Fetch(uid_t uid)
 	int asy_resolve = 0;
         repvol *vp = (repvol *)vol;
 
-	/* (Satya, 1/03)
-	Do lookaside to see if fetch can be short-cirucited.
-	Assumptions for lookaside:
-	    (a) we are operating normally (not in a repair)
-	    (b) we have valid status
-	    (c) file is a plain file (not sym link, directory, etc.)
-	    (d) non-zero SHA
-
-	(a) and (b) were  verified in Sanity Checks above;
-	(c) and (d) verified below; check for replicated volume 
-	    further ensures we never do lookaside during repair
-
-	SHA value is obtained initially from fsobj and used for lookaside;
-	If lookaside fails and a real fetch has to be done, we conservatively
-	clear this SHA value first;   this causes the fsobj's VenusSHA 
-	to also be cleared by call to UpdateStatusPlusSHA() at end;  but who 
-	cares, since we have the actual data and could always recompute
-	SHA from container before a future Venus cache replacement of the fsobj
-	*/
-	if (IsFile() && !IsZeroSHA(VenusSHA)) {
-	    int lka_successful;
-	    char emsg[256];
-	    memset(emsg, 0, sizeof(emsg));
-
-	    lka_successful = LookAsideAndFillContainer(VenusSHA, fd,
-						       stat.Length, venusRoot,
-						       emsg, sizeof(emsg)-1);
-	    if (emsg[0])
-		LOG(0, ("LookAsideAndFillContainer(%s): %s\n", cf.Name(), emsg));
-
-	    if (lka_successful) {
-		LOG(0, ("Lookaside of %s succeeded!\n", cf.Name()));
-		Recov_BeginTrans();
-		data.file->SetLength(stat.Length);
-		cf.SetValidData(cf.Length()); 
-		SetRcRights(RC_DATA | RC_STATUS); /* we now have the data */
-		code = 0; /* success */
-		Recov_EndTrans(CMFP);
-
-		/* we're done, skip out of here */
-		goto SHAExit;
-	    }
-	    /* else continue with real fetch */
-	}
-
 	/* Acquire an Mgroup. */
 	code = vp->GetMgrp(&m, uid, (PIGGYCOP2 ? &PiggyBS : 0));
 	if (code != 0) goto RepExit;
@@ -374,7 +411,7 @@ int fsobj::Fetch(uid_t uid)
 	    code = EAGAIN;
 
 	Recov_BeginTrans();
-	UpdateStatusAndSHA(&status, uid, NULL); /* CAVEAT: zero SHA being set */
+	UpdateStatus(&status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
 RepExit:
@@ -394,8 +431,6 @@ RepExit:
 	default:
 	    break;
 	}
-SHAExit:
-	/* nothing left to do */;
     }
     else {
 	/* Acquire a Connection. */
@@ -442,25 +477,25 @@ SHAExit:
 	}
 
 	Recov_BeginTrans();
-	UpdateStatusAndSHA(&status, uid, NULL); /* CAVEAT: zero SHA being set */
+	UpdateStatus(&status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
 	PutConn(&c);
     }
-    
+
     if (fd != -1)
         data.file->Close(fd);
 
+    Recov_BeginTrans();
+    RVMLIB_REC_OBJECT(flags);
+    flags.fetching = 0;
+
     if (code == 0) {
 	if (status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    SetRcRights(RC_STATUS | RC_DATA);
 
 	/* Note the presence of data. */
-	Recov_BeginTrans();
-	RVMLIB_REC_OBJECT(flags);
-	flags.fetching = 0;
-
 	switch(stat.VnodeType) {
 	case File:
 		/* File is already `truncated' to the correct length */
@@ -477,7 +512,6 @@ NonRepExit:
 	case Invalid:
 		FSO_ASSERT(this, 0);
 	}
-	Recov_EndTrans(CMFP);
     }
     else {
        /* 
@@ -485,10 +519,6 @@ NonRepExit:
 	* file length so that DiscardData releases the correct
 	* number of blocks (i.e., the number allocated in fsdb::Get).
 	*/
-	Recov_BeginTrans();
-	RVMLIB_REC_OBJECT(flags);
-	flags.fetching = 0;
-
 	/* when the server responds with EAGAIN, the VersionVector was
 	 * changed, so this should effectively be handled like a failed
 	 * validation, and we can throw away the data */
@@ -501,8 +531,8 @@ NonRepExit:
 	
 	/* Demote existing status. */
 	Demote();
-	Recov_EndTrans(CMFP);
     }
+    Recov_EndTrans(CMFP);
     return(code);
 }
 
@@ -554,7 +584,6 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
     mysha.MaxSeqLen = SHA_DIGEST_LENGTH;
     mysha.SeqLen = 0; 
 
-
     /* COP2 Piggybacking. */
     char PiggyData[COP2SIZE];
     RPC2_CountedBS PiggyBS;
@@ -611,11 +640,17 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 		 * So others that need to be checked may be missed,
 		 * resulting in more rpcs!
 		 */
-		fso_vol_iterator next(NL, vol);
-		fsobj *f = 0;
 		int numPiggyFids = 0;
 
-		while ((f = next()) && (numPiggyFids < PiggyValidations)) {
+		struct dllist_head *p;
+		list_for_each(p, vol->fso_list) {
+		    fsobj *f;
+
+		    if (numPiggyFids >= PiggyValidations)
+			break;
+
+		    f = list_entry_plusplus(p, fsobj, vol_handle);
+
 		    if (!HAVESTATUS(f) || STATUSVALID(f) || DYING(f) ||
 			FID_EQ(&f->fid, &fid) || f->IsLocalObj() || BUSY(f) ||
 			DIRTY(f))
@@ -973,7 +1008,14 @@ int fsobj::GetAttr(uid_t uid, RPC2_BoundedBS *acl)
 	}
 
 	Recov_BeginTrans();
-	UpdateStatusAndSHA(&status, uid, &mysha);
+	UpdateStatus(&status, NULL, uid);
+
+	if (IsFile()) {
+	    RVMLIB_REC_OBJECT(VenusSHA);
+	    /* VenusSHA already was set by getattr */
+	    if (mysha.SeqLen != SHA_DIGEST_LENGTH)
+		memset(&VenusSHA, 0, SHA_DIGEST_LENGTH);
+	}
 	Recov_EndTrans(CMFP);
 
 RepExit:
@@ -1002,11 +1044,24 @@ RepExit:
 		 * returns this error for all servers, we should get rid of
 		 * all cached fsobjs for this volume. */
 		if (vol) {
-		    fso_vol_iterator next(NL, vol);
-		    fsobj *f;
+		    struct dllist_head *p, *next;
 		    Recov_BeginTrans();
-		    while ((f = next()))
+		    for(p = vol->fso_list.next; p != &vol->fso_list; p = next) {
+			fsobj *n = NULL, *f;
+			f = list_entry_plusplus(p, fsobj, vol_handle);
+
+			/* Kill is scary, so we make sure we keep an active
+			 * reference to the ->next object */
+			next = p->next;
+			if (next != &vol->fso_list) {
+			    n = list_entry_plusplus(next, fsobj, vol_handle);
+			    FSO_HOLD(n);
+			}
+
 			f->Kill(0);
+
+			if (n) FSO_RELE(n);
+		    }
 		    Recov_EndTrans(CMFP);
 		}
 		break;
@@ -1086,7 +1141,14 @@ RepExit:
 	}
 
 	Recov_BeginTrans();
-	UpdateStatusAndSHA(&status, uid, &mysha); /* SHA remains zero in non-rep case */
+	UpdateStatus(&status, NULL, uid);
+
+	if (IsFile()) {
+	    RVMLIB_REC_OBJECT(VenusSHA);
+	    /* VenusSHA already was set by getattr */
+	    if (mysha.SeqLen != SHA_DIGEST_LENGTH)
+		memset(&VenusSHA, 0, SHA_DIGEST_LENGTH);
+	}
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -1117,8 +1179,13 @@ int fsobj::GetACL(RPC2_BoundedBS *acl, uid_t uid)
 	return(ETIMEDOUT);
     }
 
-    if (IsFake()) 
-	return(EINVAL);
+    if (IsFake() || IsLocalObj()) {
+	/* Just read/lookup rights for System:AnyUser */
+	const char *fakeacl = "1\n0\nSystem:AnyUser\t9\n";
+	acl->SeqLen = strlen(fakeacl) + 1;
+	memcpy(acl->SeqBody, fakeacl, acl->SeqLen);
+	return(0);
+    }
 
     /* check if the object is FETCHABLE first! */
     if (FETCHABLE(this))
@@ -1139,6 +1206,7 @@ void fsobj::LocalStore(Date_t Mtime, unsigned long NewLength)
     stat.DataVersion++;
     stat.Length = NewLength;
     stat.Date = Mtime;
+    memset(VenusSHA, 0, SHA_DIGEST_LENGTH);
 
     UpdateCacheStats((IsDir() ? &FSDB->DirAttrStats : &FSDB->FileAttrStats),
                      WRITE, NBLOCKS(sizeof(fsobj)));
@@ -1270,7 +1338,7 @@ int fsobj::ConnectedStore(Date_t Mtime, uid_t uid, unsigned long NewLength)
 	/* Do Store locally. */
 	Recov_BeginTrans();
 	LocalStore(Mtime, NewLength);
-	UpdateStatusAndClearSHA(&status, &UpdateSet, uid);
+	UpdateStatus(&status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
 
@@ -1333,7 +1401,7 @@ RepExit:
 	/* Do Store locally. */
 	Recov_BeginTrans();
 	LocalStore(Mtime, NewLength);
-	UpdateStatusAndClearSHA(&status, 0, uid);
+	UpdateStatus(&status, 0, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -1420,6 +1488,7 @@ void fsobj::LocalSetAttr(Date_t Mtime, unsigned long NewLength,
         }
         stat.Length = NewLength;
         stat.Date = Mtime;
+	memset(VenusSHA, 0, SHA_DIGEST_LENGTH);
     }
     if (NewDate != (Date_t)-1) stat.Date = NewDate;
     if (NewOwner != VA_IGNORE_UID) stat.Owner = NewOwner;
@@ -1571,7 +1640,7 @@ int fsobj::ConnectedSetAttr(Date_t Mtime, uid_t uid, unsigned long NewLength,
 	Recov_BeginTrans();
 	if (!setacl)
 		LocalSetAttr(Mtime, NewLength, NewDate, NewOwner, NewMode);
-	UpdateStatusAndClearSHA(&status, &UpdateSet, uid);
+	UpdateStatus(&status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
 
@@ -1635,7 +1704,7 @@ RepExit:
 	Recov_BeginTrans();
 	if (!setacl)
 		LocalSetAttr(Mtime, NewLength, NewDate, NewOwner, NewMode);
-	UpdateStatusAndClearSHA(&status, 0, uid);
+	UpdateStatus(&status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -1751,6 +1820,10 @@ int fsobj::SetACL(RPC2_CountedBS *acl, uid_t uid)
 	return(ETIMEDOUT);
     }
 
+    if (IsFake() || IsLocalObj())
+	/* Ignore attempts to set ACLs on fake objects. */
+	return(EPERM);
+
     struct coda_vattr va;
     va_init(&va);
     int code = SetAttr(&va, uid, acl);
@@ -1929,8 +2002,9 @@ int fsobj::ConnectedCreate(Date_t Mtime, uid_t uid, fsobj **t_fso_addr,
 	/* Do Create locally. */
 	Recov_BeginTrans();
 	LocalCreate(Mtime, target_fso, name, uid, Mode);
-	UpdateStatusAndClearSHA(&parent_status, &UpdateSet, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, &UpdateSet, uid);
+	UpdateStatus(&parent_status, &UpdateSet, uid);
+	/* New objects already have a cleared SHA */
+	target_fso->UpdateStatus(&target_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
@@ -1999,8 +2073,8 @@ RepExit:
 	/* Do Create locally. */
 	Recov_BeginTrans();
 	LocalCreate(Mtime, target_fso, name, uid, Mode);
-	UpdateStatusAndClearSHA(&parent_status, 0, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, 0, uid);
+	UpdateStatus(&parent_status, NULL, uid);
+	target_fso->UpdateStatus(&target_status, NULL, uid);
 	Recov_EndTrans(CMFP);
 	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
