@@ -186,8 +186,8 @@ int fsobj::ConnectedRemove(Date_t Mtime, uid_t uid, char *name, fsobj *target_fs
 	/* Do Remove locally. */
 	Recov_BeginTrans();
 	LocalRemove(Mtime, name, target_fso);
-	UpdateStatusAndClearSHA(&parent_status, &UpdateSet, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, &UpdateSet, uid);
+	UpdateStatus(&parent_status, &UpdateSet, uid);
+	target_fso->UpdateStatus(&target_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
 
@@ -239,8 +239,8 @@ RepExit:
 	/* Do Remove locally. */
 	Recov_BeginTrans();
 	LocalRemove(Mtime, name, target_fso);
-	UpdateStatusAndClearSHA(&parent_status, 0, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, 0, uid);
+	UpdateStatus(&parent_status, 0, uid);
+	target_fso->UpdateStatus(&target_status, 0, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -427,8 +427,8 @@ int fsobj::ConnectedLink(Date_t Mtime, uid_t uid, char *name, fsobj *source_fso)
 	/* Do Link locally. */
 	Recov_BeginTrans();
 	LocalLink(Mtime, name, source_fso);
-	UpdateStatusAndClearSHA(&parent_status, &UpdateSet, uid);
-	source_fso->UpdateStatusAndClearSHA(&source_status, &UpdateSet, uid);
+	UpdateStatus(&parent_status, &UpdateSet, uid);
+	source_fso->UpdateStatus(&source_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
 
@@ -481,8 +481,8 @@ RepExit:
 	/* Do Link locally. */
 	Recov_BeginTrans();
 	LocalLink(Mtime, name, source_fso);
-	UpdateStatusAndClearSHA(&parent_status, 0, uid);
-	source_fso->UpdateStatusAndClearSHA(&source_status, 0, uid);
+	UpdateStatus(&parent_status, NULL, uid);
+	source_fso->UpdateStatus(&source_status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -745,12 +745,12 @@ int fsobj::ConnectedRename(Date_t Mtime, uid_t uid, fsobj *s_parent_fso,
 	/* Do Rename locally. */
 	Recov_BeginTrans();
 	LocalRename(Mtime, s_parent_fso, s_name, s_fso, t_name, t_fso);
-	UpdateStatusAndClearSHA(&t_parent_status, &UpdateSet, uid);
+	UpdateStatus(&t_parent_status, &UpdateSet, uid);
 	if (!SameParent)
-		s_parent_fso->UpdateStatusAndClearSHA(&s_parent_status, &UpdateSet, uid);
-	s_fso->UpdateStatusAndClearSHA(&source_status, &UpdateSet, uid);
+		s_parent_fso->UpdateStatus(&s_parent_status, &UpdateSet, uid);
+	s_fso->UpdateStatus(&source_status, &UpdateSet, uid);
 	if (TargetExists)
-		t_fso->UpdateStatusAndClearSHA(&target_status, &UpdateSet, uid);
+		t_fso->UpdateStatus(&target_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
 
@@ -812,12 +812,12 @@ RepExit:
 	/* Do Rename locally. */
 	Recov_BeginTrans();
 	LocalRename(Mtime, s_parent_fso, s_name, s_fso, t_name, t_fso);
-	UpdateStatusAndClearSHA(&t_parent_status, 0, uid);
+	UpdateStatus(&t_parent_status, NULL, uid);
 	if (!SameParent)
-		s_parent_fso->UpdateStatusAndClearSHA(&s_parent_status, 0, uid);
-	s_fso->UpdateStatusAndClearSHA(&source_status, 0, uid);
+		s_parent_fso->UpdateStatus(&s_parent_status, NULL, uid);
+	s_fso->UpdateStatus(&source_status, NULL, uid);
 	if (TargetExists)
-		t_fso->UpdateStatusAndClearSHA(&target_status, 0, uid);
+		t_fso->UpdateStatus(&target_status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -925,7 +925,7 @@ void fsobj::LocalMkdir(Date_t Mtime, fsobj *target_fso, char *name,
 	target_fso->stat.Owner = Owner;
 	target_fso->stat.Mode = Mode;
 	target_fso->AnyUser = AnyUser;
-	memmove( (void *)target_fso->SpecificUser, (const void *)SpecificUser, (int)(CPSIZE * sizeof(AcRights)));
+	memcpy(target_fso->SpecificUser, SpecificUser, (CPSIZE * sizeof(AcRights)));
 	target_fso->Matriculate();
 	target_fso->SetParent(fid.Vnode, fid.Unique);
 
@@ -1058,8 +1058,8 @@ int fsobj::ConnectedMkdir(Date_t Mtime, uid_t uid, fsobj **t_fso_addr,
 	/* Do Mkdir locally. */
 	Recov_BeginTrans();
 	LocalMkdir(Mtime, target_fso, name, uid, Mode);
-	UpdateStatusAndClearSHA(&parent_status, &UpdateSet, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, &UpdateSet, uid);
+	UpdateStatus(&parent_status, &UpdateSet, uid);
+	target_fso->UpdateStatus(&target_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
@@ -1127,8 +1127,8 @@ RepExit:
 	/* Do Mkdir locally. */
 	Recov_BeginTrans();
 	LocalMkdir(Mtime, target_fso, name, uid, Mode);
-	UpdateStatusAndClearSHA(&parent_status, 0, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, 0, uid);
+	UpdateStatus(&parent_status, NULL, uid);
+	target_fso->UpdateStatus(&target_status, NULL, uid);
 	Recov_EndTrans(CMFP);
 	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
@@ -1373,8 +1373,8 @@ int fsobj::ConnectedRmdir(Date_t Mtime, uid_t uid, char *name, fsobj *target_fso
 	/* Do Rmdir locally. */
 	Recov_BeginTrans();
 	LocalRmdir(Mtime, name, target_fso);
-	UpdateStatusAndClearSHA(&parent_status, &UpdateSet, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, &UpdateSet, uid);
+	UpdateStatus(&parent_status, &UpdateSet, uid);
+	target_fso->UpdateStatus(&target_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (ASYNCCOP2) ReturnEarly();
 
@@ -1427,8 +1427,8 @@ RepExit:
 	/* Do Rmdir locally. */
 	Recov_BeginTrans();
 	LocalRmdir(Mtime, name, target_fso);
-	UpdateStatusAndClearSHA(&parent_status, 0, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, 0, uid);
+	UpdateStatus(&parent_status, NULL, uid);
+	target_fso->UpdateStatus(&target_status, NULL, uid);
 	Recov_EndTrans(CMFP);
 
 NonRepExit:
@@ -1526,7 +1526,7 @@ void fsobj::LocalSymlink(Date_t Mtime, fsobj *target_fso, char *name,
 	target_fso->stat.Length = linklen;
 	target_fso->data.symlink = (char *)rvmlib_rec_malloc(linklen + 1);
 	rvmlib_set_range(target_fso->data.symlink, linklen);
-	memmove(target_fso->data.symlink, contents, linklen);
+	memcpy(target_fso->data.symlink, contents, linklen);
 	UpdateCacheStats(&FSDB->FileDataStats, CREATE,
 			 NBLOCKS(linklen));
 
@@ -1655,8 +1655,8 @@ int fsobj::ConnectedSymlink(Date_t Mtime, uid_t uid, fsobj **t_fso_addr,
 	/* Do Symlink locally. */
 	Recov_BeginTrans();
 	LocalSymlink(Mtime, target_fso, name, contents, uid, Mode);
-	UpdateStatusAndClearSHA(&parent_status, &UpdateSet, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, &UpdateSet, uid);
+	UpdateStatus(&parent_status, &UpdateSet, uid);
+	target_fso->UpdateStatus(&target_status, &UpdateSet, uid);
 	Recov_EndTrans(CMFP);
 	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
@@ -1725,8 +1725,8 @@ RepExit:
 	/* Do Symlink locally. */
 	Recov_BeginTrans();
 	LocalSymlink(Mtime, target_fso, name, contents, uid, Mode);
-	UpdateStatusAndClearSHA(&parent_status, 0, uid);
-	target_fso->UpdateStatusAndClearSHA(&target_status, 0, uid);
+	UpdateStatus(&parent_status, NULL, uid);
+	target_fso->UpdateStatus(&target_status, NULL, uid);
 	Recov_EndTrans(CMFP);
 	if (target_status.CallBack == CallBackSet && cbtemp == cbbreaks)
 	    target_fso->SetRcRights(RC_STATUS | RC_DATA);
