@@ -84,6 +84,11 @@ int fsobj::Open(int writep, int execp, int truncp, venus_cnode *cp, uid_t uid)
     }
     int code = 0;
 
+    if (IsSymLink()) {
+	code = ELOOP;
+	goto Exit;
+    }
+
     /*  write lock the object if we might diddle it below.  Disabling
      * replacement and bumping reference counts are performed
      * elsewhere under read lock. */
@@ -447,8 +452,7 @@ int fsobj::Access(long rights, int modes, uid_t uid)
 
     /* !!! important point, this is where we map unauthenticated users to
      * System:AnyUser !!! */
-    userent *ue;
-    GetUser(&ue, vol->realm, uid);
+    userent *ue = vol->realm->GetUser(uid);
     int tokensvalid = ue->TokensValid();
     PutUser(&ue);
     uid_t CheckVuid = (tokensvalid ? uid : ALL_UIDS);
