@@ -63,7 +63,8 @@ extern "C" {
 
 #include "nt_util.h"
 #ifdef __CYGWIN32__
-#define main venus_main
+//  Not right now ... should go #define main venus_main
+uid_t V_UID; 
 #endif
 
 /* FreeBSD 2.2.5 defines this in rpc/types.h, all others in netinet/in.h */
@@ -83,7 +84,6 @@ int SearchForNOreFind;  // Look for better detection method for iterrupted hoard
 char *consoleFile;
 char *venusRoot;
 char *kernDevice;
-char *default_realm;
 char *realmtab;
 char *CacheDir;
 char *CachePrefix;
@@ -94,7 +94,6 @@ char *VenusPidFile;
 char *VenusControlFile;
 char *VenusLogFile;
 char *MarinerSocketPath;
-int masquerade;
 int masquerade_port;
 int PiggyValidations;
 
@@ -149,6 +148,15 @@ int main(int argc, char **argv)
     __djgpp_set_quiet_socket(1);
 #endif
 
+#ifdef __CYGWIN32__
+    /* MapPrivate does not work on Cygwin */
+    if (MapPrivate) {
+      eprint ("Private mapping turned off, does not work on CYGWIN.");
+      MapPrivate = 0;
+    }
+    V_UID = getuid();
+#endif    
+
     /* test mismatch with kernel before doing real work */
     testKernDevice();
 
@@ -380,10 +388,8 @@ static void ParseCmdline(int argc, char **argv) {
 		mariner_tcp_enable = 0;
 	    else if (STREQ(argv[i], "-allow-reattach"))
 		allow_reattach = 1;
-  	    else if (STREQ(argv[i], "-masquerade"))
-  		masquerade = 1;
-  	    else if (STREQ(argv[i], "-nomasquerade"))
-  		masquerade = 0;
+  	    else if (STREQ(argv[i], "-masquerade")) /* always on */;
+  	    else if (STREQ(argv[i], "-nomasquerade")) /* always on */;
 	    /* Private mapping ... */
 	    else if (STREQ(argv[i], "-mapprivate"))
 		MapPrivate = true;
@@ -408,37 +414,37 @@ static void DefaultCmdlineParms()
     /* Load the "venus.conf" configuration file */
     codaconf_init("venus.conf");
 
-    CONF_INT(CacheBlocks,       "cacheblocks",   40000);
-    CONF_STR(CacheDir,          "cachedir",      DFLT_CD);
-    CONF_STR(SpoolDir,          "checkpointdir", "/usr/coda/spool");
-    CONF_STR(VenusLogFile,	"logfile",	 "/usr/coda/etc/venus.log");
-    CONF_STR(consoleFile,       "errorlog",      "/usr/coda/etc/console");
-    CONF_STR(kernDevice,        "kerneldevice",  "/dev/cfs0,/dev/coda/0");
-    CONF_INT(MapPrivate,	"mapprivate",	 0);
-    CONF_STR(MarinerSocketPath, "marinersocket", "/usr/coda/spool/mariner");
-    CONF_INT(masquerade,	"masquerade",	 0);
-    CONF_INT(masquerade_port,	"masquerade_port", 0);
-    CONF_STR(venusRoot,         "mountpoint",    DFLT_VR);
-    CONF_INT(PrimaryUser,       "primaryuser",   UNSET_PRIMARYUSER);
-    CONF_STR(default_realm,     "realm",	 "testserver.coda.cs.cmu.edu");
-    CONF_STR(realmtab,          "realmtab",	 "/etc/coda/realms");
-    CONF_STR(VenusLogDevice,    "rvm_log",       "/usr/coda/LOG");
-    CONF_STR(VenusDataDevice,   "rvm_data",      "/usr/coda/DATA");
-    CONF_INT(T1Interval,	"serverprobe",   12 * 60);
+    CODACONF_INT(CacheBlocks,	    "cacheblocks",   40000);
+    CODACONF_STR(CacheDir,	    "cachedir",      DFLT_CD);
+    CODACONF_STR(SpoolDir,	    "checkpointdir", "/usr/coda/spool");
+    CODACONF_STR(VenusLogFile,	    "logfile",	     "/usr/coda/etc/venus.log");
+    CODACONF_STR(consoleFile,	    "errorlog",      "/usr/coda/etc/console");
+    CODACONF_STR(kernDevice,	    "kerneldevice",  "/dev/cfs0,/dev/coda/0");
+    CODACONF_INT(MapPrivate,	    "mapprivate",     0);
+    CODACONF_STR(MarinerSocketPath, "marinersocket", "/usr/coda/spool/mariner");
+    CODACONF_INT(masquerade_port,   "masquerade_port", 0);
+    CODACONF_STR(venusRoot,	    "mountpoint",     DFLT_VR);
+    CODACONF_INT(PrimaryUser,	    "primaryuser",    UNSET_PRIMARYUSER);
+    CODACONF_STR(realmtab,	    "realmtab",	      "/etc/coda/realms");
+    CODACONF_STR(VenusLogDevice,    "rvm_log",        "/usr/coda/LOG");
+    CODACONF_STR(VenusDataDevice,   "rvm_data",       "/usr/coda/DATA");
+
+    CODACONF_INT(T1Interval,	    "serverprobe",    150);
+    // used to be 12 minutes
 
 #if defined(__CYGWIN32__)
-    CONF_STR(CachePrefix,	"cache_prefix",	 "/?" "?/C:/cygwin");
+    CODACONF_STR(CachePrefix, "cache_prefix", "/?" "?/C:/cygwin");
 #else
     CachePrefix = "";
 #endif
 
-    CONF_INT(DontUseRVM,	"dontuservm",	 0);
+    CODACONF_INT(DontUseRVM, "dontuservm", 0);
     {
 	if (DontUseRVM)
 	    RvmType = VM;
     }
 
-    CONF_INT(CacheFiles,	"cachefiles",	 0);
+    CODACONF_INT(CacheFiles, "cachefiles", 0);
     {
 	if (!CacheFiles) CacheFiles = CacheBlocks / BLOCKS_PER_FILE;
 
@@ -452,7 +458,7 @@ static void DefaultCmdlineParms()
 	}
     }
 
-    CONF_INT(MLEs,		"cml_entries",	 0);
+    CODACONF_INT(MLEs, "cml_entries", 0);
     {
 	if (!MLEs) MLEs = CacheBlocks / BLOCKS_PER_MLE;
 
@@ -462,7 +468,7 @@ static void DefaultCmdlineParms()
 	}
     }
 
-    CONF_INT(HDBEs,		"hoard_entries", 0);
+    CODACONF_INT(HDBEs, "hoard_entries", 0);
     {
 	if (!HDBEs) HDBEs = CacheBlocks / BLOCKS_PER_HDBE;
 
@@ -473,7 +479,7 @@ static void DefaultCmdlineParms()
 	}
     }
 
-    CONF_STR(VenusPidFile,	"pid_file",	 NULL);
+    CODACONF_STR(VenusPidFile, "pid_file", NULL);
     {
 #define PIDFILE "/pid"
 	if (!VenusPidFile) {
@@ -483,7 +489,7 @@ static void DefaultCmdlineParms()
 	}
     }
 
-    CONF_STR(VenusControlFile,	"run_control_file",  NULL);
+    CODACONF_STR(VenusControlFile, "run_control_file", NULL);
     {
 #define CTRLFILE "/VENUS_CTRL"
 	if (!VenusControlFile) {
@@ -493,7 +499,7 @@ static void DefaultCmdlineParms()
 	}
     }
 
-    CONF_INT(PiggyValidations,  "validateattrs", MAX_PIGGY_VALIDATIONS);
+    CODACONF_INT(PiggyValidations, "validateattrs", 21);
     {
 	if (PiggyValidations > MAX_PIGGY_VALIDATIONS)
 	    PiggyValidations = MAX_PIGGY_VALIDATIONS;
@@ -501,7 +507,7 @@ static void DefaultCmdlineParms()
 
 #ifdef moremoremore
     char *x = NULL;
-    CONF_STR(x, "relay", NULL, "127.0.0.1");
+    CODACONF_STR(x, "relay", NULL, "127.0.0.1");
     inet_aton(x, &venus_relay_addr);
 #endif
 }
@@ -511,7 +517,7 @@ static void CdToCacheDir() {
     if (chdir(CacheDir) < 0) {
 	if (errno != ENOENT)
 	    { perror("CacheDir chdir"); exit(-1); }
-	if (mkdir(CacheDir, 0644) < 0)
+	if (mkdir(CacheDir, 0700) < 0)
 	    { perror("CacheDir mkdir"); exit(-1); }
 	if (chdir(CacheDir) < 0)
 	    { perror("CacheDir chdir"); exit(-1); }
