@@ -270,29 +270,30 @@ int VOL_HashFN(const void *key)
     return volid->Realm + volid->Volume;
 }
 
-static int GetRootVolume(Realm *realm, char **buf)
+static void GetRootVolume(Realm *realm, char **buf)
 {
     connent *c = NULL;
-    int code;
     RPC2_BoundedBS RVN;
-
-    *buf = (char *)malloc(V_MAXVOLNAMELEN);
-    if (!*buf) return ENOMEM;
-
-    memset(*buf, 0, V_MAXVOLNAMELEN);
-
-    RVN.MaxSeqLen = V_MAXVOLNAMELEN-1;
-    RVN.SeqLen = 0;
-    RVN.SeqBody = (RPC2_ByteSeq)*buf;
+    char *rvn = NULL;
+    int code;
 
     /* Get the connection. */
     if (realm->GetAdmConn(&c) != 0) {
 	LOG(100, ("GetRootVolume: can't get admin connection for realm %s!\n",
 		  realm->Name()));
 	RPCOpStats.RPCOps[ViceGetRootVolume_OP].bad++;
-	return ENOENT;
+	goto err_exit;
     }
 
+    rvn = (char *)malloc(V_MAXVOLNAMELEN);
+    if (!rvn)
+	goto err_exit;
+
+    memset(rvn, 0, V_MAXVOLNAMELEN);
+    RVN.MaxSeqLen = V_MAXVOLNAMELEN-1;
+    RVN.SeqLen = 0;
+    RVN.SeqBody = (RPC2_ByteSeq)rvn;
+
     /* Make the RPC call. */
     MarinerLog("store::GetRootVolume @%s\n", realm->Name());
     UNI_START_MESSAGE(ViceGetRootVolume_OP);
@@ -304,10 +305,18 @@ static int GetRootVolume(Realm *realm, char **buf)
 
     PutConn(&c);
 
-    LOG(10, ("GetRootVolume: (%s) received name: %s, code: %d\n",
-	     realm->Name(), *buf, code));
+    if (!code) {
+	realm->SetRootVolName(rvn);
+	LOG(10, ("GetRootVolume: (%s) received name: %s, code: %d\n",
+		 realm->Name(), rvn, code));
+    }
 
-    return code;
+err_exit:
+    if (rvn)
+	free(rvn);
+
+    *buf = strdup(realm->GetRootVolName());
+    return;
 }
 
 /* Allocate database from recoverable store. */
@@ -362,7 +371,6 @@ volent *vdb::Find(Volid *volid)
     return(0);
 }
 
-
 volent *vdb::Find(Realm *realm, const char *volname)
 {
     repvol_iterator rvnext;
@@ -461,8 +469,7 @@ volent *vdb::Create(Realm *realm, VolumeInfo *volinfo, const char *volname)
 			eprint("VSG change for volume %s [%d] %x -> %x",
 			       v->GetName(), i, vp->volreps[i], volreps[i]);
 
-			RVMLIB_REC_OBJECT(*vp->volreps[i]);
-
+			RVMLIB_REC_OBJECT(vp->volreps[i]);
 			VDB->Put((volent **)&vp->volreps[i]);
 			vp->volreps[i] = volreps[i];
 			volreps[i] = NULL;
@@ -498,10 +505,8 @@ volent *vdb::Create(Realm *realm, VolumeInfo *volinfo, const char *volname)
         break;
         }
 
-    case ROVOL: /* Readonly replicated volume are not supported */
-        break;
-
     case RWVOL:
+    case ROVOL:
     case BACKVOL:
         {
         volrep *vp;
@@ -511,7 +516,7 @@ volent *vdb::Create(Realm *realm, VolumeInfo *volinfo, const char *volname)
         /* instantiate the new volume replica */
         Recov_BeginTrans();
         vp = new volrep(realm, volinfo->Vid, volname, &srvaddr,
-                        volinfo->Type == BACKVOL,
+                        volinfo->Type != RWVOL,
                         (&volinfo->Type0)[replicatedVolume]);
         v = vp;
         Recov_EndTrans(MAXFP);
@@ -573,7 +578,6 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
     char *realm_name = NULL;
     Realm *realm;
     char *volname = strdup(name);
-    char *volinfoname = NULL;
     CODA_ASSERT(volname);
 
     SplitRealmFromName(volname, &realm_name);
@@ -597,10 +601,10 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
 
     /* "special" case? not everybody wants to use the pathname based volume
      * name mapping and uses a rootvolume named "/" */
-    if (volname[0] == '\0' || (volname[0] == '/' && volname[1] == '\0'))
-	code = GetRootVolume(realm, &volinfoname);
-    if (code || !volinfoname)
-	volinfoname = strdup(volname);
+    if (volname[0] == '\0') {
+	free(volname);
+	GetRootVolume(realm, &volname);
+    }
 
     LOG(100, ("vdb::Get: volname = %s@%s\n", volname, realm->Name()));
 
@@ -617,7 +621,7 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
 	/* Make the RPC call. */
 	MarinerLog("store::GetVolumeInfo %s@%s\n", volname, realm->Name());
 	UNI_START_MESSAGE(ViceGetVolumeInfo_OP);
-	code = (int) ViceGetVolumeInfo(c->connid, (RPC2_String)volinfoname, &volinfo);
+	code = (int) ViceGetVolumeInfo(c->connid, (RPC2_String)volname, &volinfo);
 	UNI_END_MESSAGE(ViceGetVolumeInfo_OP);
 	MarinerLog("store::getvolumeinfo done\n");
 
@@ -640,7 +644,10 @@ int vdb::Get(volent **vpp, Realm *prealm, const char *name, fsobj *f)
 
 	goto error_exit;
     }
-    if (v && v->GetVolumeId() != volinfo.Vid) {
+    if (v) {
+	if (v->GetVolumeId() == volinfo.Vid)
+	    goto Exit;
+
 	eprint("Mapping changed for volume %s@%s (%x --> %x)",
 	       volname, realm->Name(), v->GetVolumeId(), volinfo.Vid);
 
@@ -683,8 +690,6 @@ Exit:
 
 error_exit:
     realm->PutRef();
-    if (volinfoname)
-	free(volinfoname);
     if (volname)
 	free(volname);
     return code;
@@ -914,7 +919,7 @@ void volent::ResetVolTransients()
     flags.weaklyconnected = 0;
     flags.available = 1;
 
-    fso_list = new olist;
+    list_head_init(&fso_list);
 
     /* 
      * sync doesn't need to be initialized. 
@@ -932,9 +937,8 @@ volent::~volent()
 
     /* Drain and delete transient lists. */
     {
-	if (fso_list->count() != 0)
+	if (!list_empty(&fso_list))
 	    CHOKE("volent::~volent: fso_list not empty");
-	delete fso_list;
     }
 
     realm->Rec_PutRef();
@@ -952,14 +956,37 @@ void volent::operator delete(void *deadobj, size_t len)
     rvmlib_rec_free(deadobj);
 }
 
-void volent::hold() {
+void volent::hold(void)
+{
     refcnt++;
 }
 
-void volent::release() {
+void volent::release(void)
+{
+    int intrans;
+
     refcnt--;
 
     CODA_ASSERT(refcnt >= 0);
+
+    if (refcnt > 0)
+	return;
+
+    /* special test needed for volumes with local repair conflict */
+    if (IsReplicated() && ((repvol *)this)->CML.count() != 0)
+	return;
+
+    /* we could be called when there already is a transaction ongoing or we
+     * might have to start our own transaction... */
+    intrans = rvmlib_in_transaction();
+    if (!intrans)
+	Recov_BeginTrans();
+
+    if (IsReplicated()) delete (repvol *)this;
+    else	        delete (volrep *)this;
+
+    if (!intrans)
+	Recov_EndTrans(MAXFP);
 }
 
 int volent::IsReadWriteReplica()
@@ -1013,11 +1040,12 @@ int volent::Enter(int mode, uid_t uid)
 
         if (IsReplicated()) ((repvol *)this)->ClearCallBack();
 
-        fso_vol_iterator next(NL, this);
-        fsobj *f;
-        while ((f = next()))
-            f->Demote();
-
+	struct dllist_head *p;
+	list_for_each(p, fso_list) {
+	    fsobj *f = list_entry_plusplus(p, fsobj, vol_handle);
+	    /* demote does not yield or delete the object */
+	    f->Demote();
+	}
         just_transitioned = 1;
     } 
 
@@ -1210,10 +1238,12 @@ void volent::Exit(int mode, uid_t uid)
 
         if (IsReplicated()) ((repvol *)this)->ClearCallBack();
 
-	fso_vol_iterator next(NL, this);
-	fsobj *f;
-	while ((f = next()))
+	struct dllist_head *p;
+	list_for_each(p, fso_list) {
+	    fsobj *f = list_entry_plusplus(p, fsobj, vol_handle);
+	    /* demote does not yield or delete the object */
 	    f->Demote();
+	}
     }
 
     /* Step 2 is to "exit" this volume. */
@@ -1330,8 +1360,7 @@ void volent::TakeTransition()
      * 3. We force "zombie" volumes to emulation state until they are
      *    un-zombied. */
     if (nextstate == Logging && rv->GetCML()->count() > 0) {
-	userent *u = 0;
-	GetUser(&u, rv->realm, rv->GetCML()->Owner());
+	userent *u = rv->realm->GetUser(rv->GetCML()->Owner());
 	if (!u->TokensValid()) {
 	    rv->SetReintegratePending();
 	    nextstate = Emulating;
@@ -1655,8 +1684,7 @@ void repvol::SetReintegratePending() {
 void repvol::ClearReintegratePending() {
     flags.reintegratepending = 0;
     /* if (SkkEnabled) {
-     *    userent *u;
-     *    GetUser(&u, realm, CML.owner);
+     *    userent *u = realm->GetUser(CML.owner);
      *    CODA_ASSERT(u != NULL);
      *    u->NotifyReintegrationEnabled(name);
      * } */
@@ -1667,8 +1695,7 @@ void repvol::CheckReintegratePending() {
     if (flags.reintegratepending && CML.count() > 0) {
         eprint("Reintegrate %s pending tokens for uid = %d", name, CML.owner);
 	/* if (SkkEnabled) {
-         *    userent *u;
-         *    GetUser(&u, realm, CML.owner);
+         *    userent *u = realm->GetUser(CML.owner);
          *    CODA_ASSERT(u != NULL);
          *    u->NotifyReintegrationPending(name);
 	 * } */
@@ -2407,7 +2434,7 @@ VenusFid repvol::GenerateLocalFid(ViceDataType fidtype)
 
 
 /* MUST be called from within a transaction */
-VenusFid repvol::GenerateFakeFid() 
+VenusFid repvol::GenerateFakeFid()
 {
     VenusFid fid;
     FID_MakeSubtreeRoot(MakeViceFid(&fid), vid, FidUnique);
@@ -2724,11 +2751,16 @@ void volent::GetMountPath(char *buf, int ok_to_assert)
 
 void volent::print(int afd)
 {
+    struct dllist_head *p;
+    int nfsos = 0;
+
+    list_for_each(p, fso_list) { nfsos++; }
+
     fdprint(afd, "%#08x : %-16s : vol = %x @%s\n", (long)this, name, vid,
 	    realm->Name());
 
     fdprint(afd, "\trefcnt = %d, fsos = %d, logv = %d, weak = %d\n",
-	    refcnt, fso_list->count(), flags.logv, flags.weaklyconnected);
+	    refcnt, nfsos, flags.logv, flags.weaklyconnected);
     fdprint(afd, "\tstate = %s, t_p = %d, d_p = %d, counts = [%d %d %d %d], repair = %d\n",
 	    PRINT_VOLSTATE(state), flags.transition_pending, flags.demotion_pending,
 	    observer_count, mutator_count, waiter_count, resolver_count, 
@@ -2792,13 +2824,11 @@ void volent::ListCache(FILE* fp, int long_format, unsigned int valid)
     char mountpath[MAXPATHLEN];
     GetMountPath(mountpath, 0);
     fprintf(fp, "%s: %lx\n", mountpath, vid);
-    fsobj *f = 0;
-    fso_vol_iterator next(NL, this);
-    while ((f = next())) {
-        if ( !FSDB->Find(&f->fid) )
-            fprintf(fp, "Not Cached.");
-        else 
-            f->ListCache(fp, long_format, valid);
+
+    struct dllist_head *p;
+    list_for_each(p, fso_list) {
+	fsobj *f = list_entry_plusplus(p, fsobj, vol_handle);
+	f->ListCache(fp, long_format, valid);
     }
     fprintf(fp, "\n");
     fflush(fp);
