@@ -453,7 +453,7 @@ void TrickleReintegrate(); /* used to be in class vdb (Satya 5/20/95) */
 
 /* Volume Database.  Dictionary for volume entries (volents). */
 class vdb {
-  friend void VolInit();
+  friend void VolInit(void);
   friend void VOLD_Init(void);
   friend void VolDaemon(void);
   friend class cmlent;
@@ -523,6 +523,7 @@ class vdb {
     void print(int, int =0);
 
     void ListCache(FILE *, int long_format=1, unsigned int valid=3);
+    int FreeMLECount(void) { return MaxMLEs - AllocatedMLEs; }
 };
 
 
@@ -589,7 +590,6 @@ class volent {
   friend class fsobj;
   friend class userent;
   friend class vdb;
-  friend class fso_vol_iterator;
   friend class repvol_iterator;
   friend class volrep_iterator;
   friend class vproc;                /* flags.autowriteback */
@@ -621,7 +621,7 @@ class volent {
     rec_olink handle;			/* link for repvol_hash/volrep_hash */
 
     /* Fso's. */
-    /*T*/olist *fso_list;
+    /*T*/struct dllist_head fso_list;
 
     /* State information. */
     /*T*/short mutator_count;
@@ -689,6 +689,7 @@ class srvent;
 /* A volume replica entry. */
 class volrep : public volent {
     friend class vdb;
+    friend class volent;
     friend void VolInit(void);
 
     VolumeId replicated;        /* replicated `parent' volume */
