@@ -90,7 +90,7 @@ void repvol::Reintegrate()
 	return;
 
 #if 0
-    GetUser(&u, realm, CML.owner);
+    u = realm->GetUser(CML.owner);
     CODA_ASSERT(u != NULL);
     /* if (SkkEnabled) u->NotifyReintegrationActive(name); */
     PutUser(&u);
@@ -568,9 +568,8 @@ CheckResult:
 int repvol::ReadyToReintegrate()
 {
     int ready = 0;
-    userent *u = 0;
+    userent *u = realm->GetUser(CML.owner); /* if CML is non-empty, u != 0 */
 
-    GetUser(&u, realm, CML.owner); /* if the CML is non-empty, u != 0 */
     /* 
      * we're a bit draconian about ASRs.  We want to avoid reintegrating
      * while an ASR is in progress, because the ASR uses the write
