@@ -245,7 +245,7 @@ void vproc::do_ioctl(VenusFid *fid, unsigned int com, struct ViceIoctl *data) {
 
 			/*
 			 * Regular mount-links start with a '#', optionally
-			 * followed by a volume name (GetRootVolume is used in
+			 * followed by a volume name (current path is used in
 			 * case the volume name is not specified), optionally
 			 * followed by '@' and a realm/domain name (the realm
 			 * of the parent volume is used if this is not
@@ -363,7 +363,8 @@ void vproc::do_ioctl(VenusFid *fid, unsigned int com, struct ViceIoctl *data) {
 			if (u.u_error) { FSDB->Put(&target_fso); break; }
 
 			/* Retrieve the link contents from the cache. */
-			u.u_error = target_fso->Readlink((char *)data->out, MAXPATHLEN,
+			u.u_error = target_fso->Readlink((char *)data->out,
+							 CODA_MAXPATHLEN,
 							 &out_size, u.u_uid);
 			if (u.u_error) {
                             FSDB->Put(&target_fso);
@@ -401,7 +402,7 @@ void vproc::do_ioctl(VenusFid *fid, unsigned int com, struct ViceIoctl *data) {
 		        }
 
 			/* Copy out the parent fid. */
-			memmove((void *) data->out, (const void *)&f->pfid, (int)sizeof(VenusFid));
+			memcpy(data->out, &f->pfid, sizeof(VenusFid));
 			data->out_size = (short)sizeof(VenusFid);
 
 			break;
@@ -519,7 +520,7 @@ O_FreeLocks:
 		    memcpy(&cp->vv, &f->stat.VV, sizeof(ViceVersionVector));
 
 		    strcpy(cp->realm, f->vol->realm->Name());
-		    
+
 		    data->out_size = sizeof(struct GetFid);
 		    break;
 		    }
@@ -753,7 +754,7 @@ OI_FreeLocks:
 		    /* Copy all the junk back out. */
 		    /* Format is (status, name, offlinemsg, motd). */
 		    cp = (char *) data->out;
-		    memmove(cp, (char *)&volstat, (int)sizeof(VolumeStatus));
+		    memcpy(cp, &volstat, sizeof(VolumeStatus));
 		    cp += sizeof(VolumeStatus);
 		    strcpy(cp, name);
 		    cp += strlen(name) + 1;
@@ -1155,14 +1156,15 @@ V_FreeLocks:
 			*clearlen != (int)sizeof(ClearToken))
 			{ u.u_error = EINVAL; break; }
 */
-		    userent *ue;
-		    Realm *realm;
-		    realm = REALMDB->GetRealm(realmp);
-		    GetUser(&ue, realm, u.u_uid);
-		    u.u_error = 0;
-		    if (!ue->SetTokens(secretp, clearp))
-			u.u_error = EPERM;
-		    PutUser(&ue);
+		    Realm *realm = REALMDB->GetRealm(realmp);
+
+		    u.u_error = realm->NewUserToken(u.u_uid, secretp, clearp);
+		    if (!u.u_error) {
+			connent *c = NULL;
+			/* attempt to trigger reintegration */
+			realm->GetAdmConn(&c);
+			PutConn(&c);
+		    }
 		    realm->PutRef();
 #undef	secretlen
 #undef	secretp
@@ -1181,11 +1183,9 @@ V_FreeLocks:
 #define clearlen ((long *)(secretp + 1))
 #define clearp ((ClearToken *)(clearlen + 1))
 #define endp ((char *)(clearp + 1)) 
-		    userent *ue;
-		    Realm *realm;
-		    realm = REALMDB->GetRealm(data->in);
-		    GetUser(&ue, realm, u.u_uid);	   
-		    u.u_error = (int) ue->GetTokens(secretp, clearp);
+		    Realm *realm = REALMDB->GetRealm(data->in);
+		    userent *ue = realm->GetUser(u.u_uid);	   
+		    u.u_error = ue->GetTokens(secretp, clearp);
 		    PutUser(&ue);
 		    realm->PutRef();
 		    if (u.u_error) break;
@@ -1203,9 +1203,8 @@ V_FreeLocks:
 
 		case VIOCUNLOG:
 		    {
-		    userent *ue;
 		    Realm *realm = REALMDB->GetRealm(data->in);
-		    GetUser(&ue, realm, u.u_uid);
+		    userent *ue = realm->GetUser(u.u_uid);
 		    ue->Invalidate();
 		    PutUser(&ue);
 		    realm->PutRef();
@@ -1388,9 +1387,8 @@ V_FreeLocks:
 		     * cooperation with the kernel, which I don't want to mess
 		     * with now.  So instead, we will set it on a per-user
 		     * basis (at least for now). */
-		    userent *ue;
 		    Realm *realm = REALMDB->GetRealm((char *)data->in+sizeof(int));
-		    GetUser(&ue, realm, u.u_uid);
+		    userent *ue = realm->GetUser(u.u_uid);
 		    ue->SetWaitForever(on);
 		    PutUser(&ue);
 		    realm->PutRef();
@@ -1407,7 +1405,7 @@ V_FreeLocks:
 		    char *realmname = (char *)data->in + sizeof(ViceFid);
 		    Realm *realm = REALMDB->GetRealm(realmname);
 
-		    int	out_size = MAXPATHLEN;	    /* needed since data->out_size is a short! */
+		    int	out_size = CODA_MAXPATHLEN;	    /* needed since data->out_size is a short! */
 		    VenusFid vfid;
 		    MakeVenusFid(&vfid, realm->Id(), &fid);
 
@@ -1622,7 +1620,7 @@ V_FreeLocks:
 			    {  
 				/* list local mutations belonging to this session */
 				int dummy;
-				char fpath[MAXNAMELEN];
+				char fpath[CODA_MAXPATHLEN];
 				sscanf((char *) data->in, "%d %s", &dummy, fpath);
 				if (LRDB->repair_root_fid == NULL) {
 				    sprintf((char *) data->out, "no repair session going on\n");
