@@ -445,7 +445,7 @@ long FS_ViceSetVV(RPC2_Handle cid, ViceFid *Fid, ViceVersionVector *VV, RPC2_Cou
 	    goto FreeLocks;
 	}
     }
-    memmove((void *) &(Vnode_vv(vptr)), (const void *)VV, (int)sizeof(ViceVersionVector));
+    memcpy(&(Vnode_vv(vptr)), VV, sizeof(ViceVersionVector));
     
 FreeLocks:
     rvmlib_begin_transaction(restore);
@@ -950,9 +950,8 @@ int SetRights(Vnode *vptr, char *name, int rights)
 	    else {
 		/* remove this entry since rights are zero */
 		for (int j = i; j < (aCL->PlusEntriesInUse - 1); j++)
-		    memmove((void *) &(aCL->ActualEntries[j]),
-			    (const void *)&(aCL->ActualEntries[j+1]),
-			    (int) sizeof(AL_AccessEntry));
+		    memcpy(&(aCL->ActualEntries[j]), &(aCL->ActualEntries[j+1]),
+			   sizeof(AL_AccessEntry));
 		aCL->PlusEntriesInUse--;
 		aCL->TotalNoOfEntries--;
 		aCL->MySize -= (int) sizeof(AL_AccessEntry);
@@ -967,9 +966,8 @@ int SetRights(Vnode *vptr, char *name, int rights)
 	for (int i = aCL->TotalNoOfEntries - 1; 
 	     i > (aCL->TotalNoOfEntries - aCL->MinusEntriesInUse - 1); 
 	     i--)
-	    memmove((void *) &(aCL->ActualEntries[i+1]),
-		    (const void *)&(aCL->ActualEntries[i]),
-		    (int) sizeof(AL_AccessEntry));
+	    memcpy(&(aCL->ActualEntries[i+1]), &(aCL->ActualEntries[i]),
+		   sizeof(AL_AccessEntry));
 	aCL->TotalNoOfEntries++;
 	aCL->MySize += (int) sizeof(AL_AccessEntry);
     }
@@ -1015,9 +1013,8 @@ int SetNRights(Vnode *vptr, char *name, int rights)
 	    else {
 		/* remove this entry since rights are zero */
 		for (int j = i; j > t - m; j--)
-		    memmove((void *) &(aCL->ActualEntries[j]),
-			    (const void *)&(aCL->ActualEntries[j-1]), 
-			    (int) sizeof(AL_AccessEntry));
+		    memcpy(&(aCL->ActualEntries[j]), &(aCL->ActualEntries[j-1]),
+			   sizeof(AL_AccessEntry));
 		aCL->MinusEntriesInUse--;
 		aCL->TotalNoOfEntries--;
 		aCL->MySize -= (int) sizeof(AL_AccessEntry);
@@ -1029,9 +1026,8 @@ int SetNRights(Vnode *vptr, char *name, int rights)
     if ((m + p) == t){
 	/* all entries are taken - create one */
 	for (int i = t - 1; i > t - m - 1; i--)
-	    memmove((void *) &(aCL->ActualEntries[i+1]),
-		    (const void *)&(aCL->ActualEntries[i]), 
-		    (int) sizeof(AL_AccessEntry));
+	    memcpy(&(aCL->ActualEntries[i+1]), &(aCL->ActualEntries[i]),
+		   sizeof(AL_AccessEntry));
 	t = ++aCL->TotalNoOfEntries;
 	aCL->MySize += (int)sizeof(AL_AccessEntry);
     }
@@ -1292,14 +1288,16 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		if ((errorCode = AllocVnode(&cv->vptr, volptr, (ViceDataType)vFile, &cFid,
 					   &(ov->fid), client->Id, 1, &tblocks)))
 		    return(errorCode);
+
 		*deltablocks += tblocks;
 		tblocks = 0;
 
 		/* add name to parent */
-		PerformCreate(client, VSGVolnum, volptr, ov->vptr,
-			      cv->vptr, repairent.name, status->Date,
-			      status->Mode, 0, StoreId, 
-			      &ov->d_cinode, &tblocks);
+		errorCode = PerformCreate(client, VSGVolnum, volptr, ov->vptr,
+					  cv->vptr, repairent.name,
+					  status->Date, 0644, 0, StoreId,
+					  &ov->d_cinode, &tblocks);
+		CODA_ASSERT(errorCode == 0);
 		*deltablocks += tblocks;
 
 		/* create the inode */
@@ -1329,10 +1327,11 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		tblocks = 0;
 
 		/* make the child directory and insert name in parent */
-		PerformMkdir(client, VSGVolnum, volptr, ov->vptr,
-			     cv->vptr, repairent.name, status->Date,
-			     status->Mode, 0, StoreId, 
-			     &ov->d_cinode, &tblocks);
+		errorCode = PerformMkdir(client, VSGVolnum, volptr, ov->vptr,
+					 cv->vptr, repairent.name,
+					 status->Date, 0755, 0, StoreId,
+					 &ov->d_cinode, &tblocks);
+		CODA_ASSERT(errorCode == 0);
 		*deltablocks += tblocks;
 
 		/* set the delete flag to true - for abort case */
@@ -1353,10 +1352,11 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		tblocks = 0;
 
 		/* create the symlink */
-		PerformSymlink(client, VSGVolnum, volptr, ov->vptr,
-			       cv->vptr, repairent.name, 0, 0, 
-			       status->Date, status->Mode,
-			       0, StoreId, &ov->d_cinode, &tblocks);
+		errorCode = PerformSymlink(client, VSGVolnum, volptr, ov->vptr,
+					   cv->vptr, repairent.name, 0, 0,
+					   status->Date, 0777, 0, StoreId,
+					   &ov->d_cinode, &tblocks);
+		CODA_ASSERT(errorCode == 0);
 		*deltablocks += tblocks;
 		
 		/* create the inode */
@@ -1387,9 +1387,10 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 		}
 
 		/* add name to parent */
-		PerformLink(client, VSGVolnum, volptr, ov->vptr, cv->vptr,
-			    repairent.name, status->Date, 0, StoreId, 
-			    &ov->d_cinode, &tblocks);
+		errorCode = PerformLink(client, VSGVolnum, volptr, ov->vptr,
+					cv->vptr, repairent.name, status->Date,
+					0, StoreId, &ov->d_cinode, &tblocks);
+		CODA_ASSERT(errorCode == 0);
 		*deltablocks += tblocks;
 	    }
 	    break;
@@ -1570,16 +1571,14 @@ static int PerformDirRepair(ClientEntry *client, vle *ov, Volume *volptr,
 	}
     }
     
-    /* set status of directory being repaired */
+    /* update VV (and status) of directory being repaired */
     {
-	ov->vptr->disk.author = status->Author;
-	ov->vptr->disk.unixModifyTime = status->Date;
-	ov->vptr->disk.modeBits = status->Mode;
 	ViceVersionVector DiffVV;
 	DiffVV = status->VV;
 	SubVVs(&DiffVV, &Vnode_vv(ov->vptr));
 	AddVVs(&Vnode_vv(ov->vptr), &DiffVV);
  	AddVVs(&V_versionvector(volptr), &DiffVV); 
+	ov->vptr->disk.author = client->Id;
 	NewCOP1Update(volptr, ov->vptr, StoreId);
 	SetStatus(ov->vptr, status, rights, anyrights);
     }
