@@ -175,7 +175,7 @@ ReadConfigFile()
     /* Load configuration file to get vice dir. */
     codaconf_init("server.conf");
 
-    CONF_STR(vicedir, "vicedir", "/vice");
+    CODACONF_STR(vicedir, "vicedir", "/vice");
 
     vice_dir_init(vicedir, 0);
 }
@@ -1091,7 +1091,7 @@ static void VUInitServerList() {
 	    } else {
 		long netaddress;
 		CODA_ASSERT(hostent->h_length == 4);
-		memmove((char *)&netaddress, (char *)hostent->h_addr, 4);
+		memcpy(&netaddress, hostent->h_addr, 4);
 		HostAddress[sid] = Hosts[sid].address = ntohl(netaddress);
 		strncpy(Hosts[sid].name, hostent->h_name, 100);
 	    }
