@@ -237,7 +237,8 @@ long S_VolMakeBackups(RPC2_Handle rpcid, VolumeId originalId,
 	V_backupDate(originalvp) = time(0);
 
 	/* Put the RW's VVV in the backupvp so vol-dump can create an ordering of dumps. */
-	memmove((void *) &V_versionvector(backupvp), (const void *)&V_versionvector(originalvp), sizeof(vv_t));
+	memcpy(&V_versionvector(backupvp), &V_versionvector(originalvp),
+	       sizeof(vv_t));
 	
 	*backupId = V_backupId(originalvp);
     }
@@ -709,7 +710,6 @@ static void updateBackupVnodes(Volume *rwvp, Volume *backupvp,
 		    rwVnode->cloned = 0;	// backupVnodes can't be cloned.
 		    rwVnode->versionvector.Flags = 0; // R/O vnodes shouldn't be inconsistent.
 		    rwVnode->vol_index = bvdop->vol_index;
-		    rwVnode->lock = bvdop->lock;	// Use the backupVnode's lock
 		    rwVnode->nextvn = bvdop->nextvn;
 
 		    rvmlib_modify_bytes(bvdop, rwVnode, (vclass == vLarge) ?
