@@ -282,8 +282,8 @@ struct command cmdarray[] =
             NULL
         },
         {"listvol", "lv", ListVolume, 
-            "cfs listvol <dir> [<dir> <dir> ...]",
-            "Display volume status",
+            "cfs listvol [-local] <dir> [<dir> <dir> ...]",
+            "Display volume status (-local avoids querying server)",
             NULL
         },
         {"lookaside", "lka", LookAside, 
@@ -479,7 +479,7 @@ int main(int argc, char *argv[])
     }
 
     codaconf_init("venus.conf");
-    CONF_STR(mountpoint, "mountpoint", "/coda");
+    CODACONF_STR(mountpoint, "mountpoint", "/coda");
 
     /* invoke the handler */
     ((PFV3)cmdarray[slot].handler)(argc, argv, slot);
@@ -914,7 +914,7 @@ static int findclosures(char ***clist)
     DIR *dirp;
     struct dirent *td;
 
-    CONF_STR(checkpointdir, "checkpointdir", "/usr/coda/spool");
+    CODACONF_STR(checkpointdir, "checkpointdir", "/usr/coda/spool");
     sprintf(spooldir, "%s/%d", checkpointdir, getuid());
 
     dirp = opendir(spooldir);
@@ -1232,7 +1232,7 @@ static int pioctl_GetFid(char *path, ViceFid *fid, char *realm, ViceVersionVecto
         ViceFid           fid;
         ViceVersionVector vv;
 	char		  realm[MAXHOSTNAMELEN+1];
-    }                out;
+    } out;
     struct ViceIoctl vio;
     int              rc;
 
@@ -1805,7 +1805,6 @@ static void ListVolume(int argc, char *argv[], int opslot)
         exit(-1);
         }
 
-
     for (i = 2; i < argc; i++)
         {
 	if (strcmp(argv[i], "-local") == 0) {
@@ -1850,7 +1849,8 @@ static void ListVolume(int argc, char *argv[], int opslot)
         if (*motd) printf("  Message of the day is \"%s\"\n", motd);
         printf("  Volume type is %s\n", xlate_vvtype(vs->Type));
 	printf("  Connection State is %s\n", print_conn_state(conn_state));
-	if (conn_state!=Emulating) { /* info not avail if disconnected */
+	/* info not avail if disconnected, or if we did a local query */
+	if (conn_state!=Emulating && local_only == 0) {
 	    printf("  Minimum quota is %lu,", vs->MinQuota);
 	    if (vs->MaxQuota > 0)
 		printf(" maximum quota is %lu\n", vs->MaxQuota);
