@@ -30,6 +30,7 @@ extern "C" {
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/time.h>
@@ -49,6 +50,7 @@ extern "C" {
 #include <lwp/lwp.h>
 #include <lwp/timer.h>
 #include <rpc2/rpc2.h>
+#include <rpc2/rpc2_addrinfo.h>
 #include <rpc2/se.h>
 #include <rpc2/sftp.h>
 #include <signal.h>
@@ -361,14 +363,13 @@ BadArgs:
 
 
 static void InitRPC()
-    {
+{
+    RPC2_Options options;
     int pid;
     int rc;
     SFTP_Initializer sei;
     struct timeval tv;
 
-
-
     /* Init RPC2 */
     rc = LWP_Init(LWP_VERSION, LWP_NORMAL_PRIORITY, (PROCESS *)&pid);
     if (rc != LWP_SUCCESS) 
@@ -378,8 +379,12 @@ static void InitRPC()
     SFTP_Activate(&sei);
     tv.tv_sec = 15;
     tv.tv_usec = 0;
-    RPC2_Init(RPC2_VERSION, 0, 0, -1, &tv);
-    }
+
+    memset(&options, 0, sizeof(options));
+    options.Flags = RPC2_OPTION_IPV6;
+
+    RPC2_Init(RPC2_VERSION, &options, 0, -1, &tv);
+}
 
 
 static void DrawCaptions()
@@ -662,13 +667,20 @@ int CmpDisk(ViceDisk **d1, ViceDisk **d2)
     }
 
 static int ValidServer(char *s)
-    {
-    struct hostent *he;
+{
+    struct RPC2_addrinfo hints, *res = NULL;
+    int ret;
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family   = PF_INET;
+    hints.ai_socktype = SOCK_DGRAM;
+    hints.ai_protocol = IPPROTO_UDP;
+
+    ret = RPC2_getaddrinfo(s, "codasrv", &hints, &res);
+    RPC2_freeaddrinfo(res);
     
-    he = gethostbyname(s);
-    if (he) return(1);
-    else return(0);
-    }
+    return (ret == 0);
+}
 
 static char *ShortDiskName(char *s)
     /* Returns 5-char abbreviation of a disk name returned
